# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Dataset(pulumi.CustomResource):
    accesses: pulumi.Output[list]
    """
    An array of objects that define dataset access for one or more entities.  Structure is documented below.

      * `domain` (`str`) - A domain to grant access to. Any users signed in with the
        domain specified will be granted the specified access
      * `group_by_email` (`str`) - An email address of a Google Group to grant access to.
      * `role` (`str`) - Describes the rights granted to the user specified by the other
        member of the access object. Primitive, Predefined and custom
        roles are supported. Predefined roles that have equivalent
        primitive roles are swapped by the API to their Primitive
        counterparts. See
        [official docs](https://cloud.google.com/bigquery/docs/access-control).
      * `special_group` (`str`) - A special group to grant access to. Possible values include:
      * `user_by_email` (`str`) - An email address of a user to grant access to. For example:
        fred@example.com
      * `view` (`dict`) - A view from a different dataset to grant access to. Queries
        executed against that view will have read access to tables in
        this dataset. The role field is not required when this field is
        set. If that view is updated by any user, access to the view
        needs to be granted again via an update operation.  Structure is documented below.
        * `dataset_id` (`str`) - The ID of the dataset containing this table.
        * `project_id` (`str`) - The ID of the project containing this table.
        * `table_id` (`str`) - The ID of the table. The ID must contain only letters (a-z,
          A-Z), numbers (0-9), or underscores (_). The maximum length
          is 1,024 characters.
    """
    creation_time: pulumi.Output[float]
    """
    The time when this dataset was created, in milliseconds since the epoch.
    """
    dataset_id: pulumi.Output[str]
    """
    The ID of the dataset containing this table.
    """
    default_encryption_configuration: pulumi.Output[dict]
    """
    The default encryption key for all tables in the dataset. Once this property is set,
    all newly-created partitioned tables in the dataset will have encryption key set to
    this value, unless table creation request (or query) overrides the key.  Structure is documented below.

      * `kms_key_name` (`str`) - Describes the Cloud KMS encryption key that will be used to protect destination
        BigQuery table. The BigQuery Service Account associated with your project requires
        access to this encryption key.
    """
    default_partition_expiration_ms: pulumi.Output[float]
    """
    The default partition expiration for all partitioned tables in
    the dataset, in milliseconds.
    """
    default_table_expiration_ms: pulumi.Output[float]
    """
    The default lifetime of all tables in the dataset, in milliseconds.
    The minimum value is 3600000 milliseconds (one hour).
    """
    delete_contents_on_destroy: pulumi.Output[bool]
    """
    If set to `true`, delete all the tables in the
    dataset when destroying the resource; otherwise,
    destroying the resource will fail if tables are present.
    """
    description: pulumi.Output[str]
    """
    A user-friendly description of the dataset
    """
    etag: pulumi.Output[str]
    """
    A hash of the resource.
    """
    friendly_name: pulumi.Output[str]
    """
    A descriptive name for the dataset
    """
    labels: pulumi.Output[dict]
    """
    The labels associated with this dataset. You can use these to
    organize and group your datasets
    """
    last_modified_time: pulumi.Output[float]
    """
    The date when this dataset or any of its tables was last modified, in milliseconds since the epoch.
    """
    location: pulumi.Output[str]
    """
    The geographic location where the dataset should reside.
    See [official docs](https://cloud.google.com/bigquery/docs/dataset-locations).
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    def __init__(__self__, resource_name, opts=None, accesses=None, dataset_id=None, default_encryption_configuration=None, default_partition_expiration_ms=None, default_table_expiration_ms=None, delete_contents_on_destroy=None, description=None, friendly_name=None, labels=None, location=None, project=None, __props__=None, __name__=None, __opts__=None):
        """
        Datasets allow you to organize and control access to your tables.


        To get more information about Dataset, see:

        * [API documentation](https://cloud.google.com/bigquery/docs/reference/rest/v2/datasets)
        * How-to Guides
            * [Datasets Intro](https://cloud.google.com/bigquery/docs/datasets-intro)

        ## Example Usage

        ### Bigquery Dataset Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        bqowner = gcp.service_account.Account("bqowner", account_id="bqowner")
        dataset = gcp.bigquery.Dataset("dataset",
            dataset_id="example_dataset",
            friendly_name="test",
            description="This is a test description",
            location="EU",
            default_table_expiration_ms=3600000,
            labels={
                "env": "default",
            },
            access=[
                {
                    "role": "OWNER",
                    "user_by_email": bqowner.email,
                },
                {
                    "role": "READER",
                    "domain": "hashicorp.com",
                },
            ])
        ```

        ### Bigquery Dataset Cmek

        ```python
        import pulumi
        import pulumi_gcp as gcp

        key_ring = gcp.kms.KeyRing("keyRing", location="us")
        crypto_key = gcp.kms.CryptoKey("cryptoKey", key_ring=key_ring.id)
        dataset = gcp.bigquery.Dataset("dataset",
            dataset_id="example_dataset",
            friendly_name="test",
            description="This is a test description",
            location="US",
            default_table_expiration_ms=3600000,
            default_encryption_configuration={
                "kms_key_name": crypto_key.id,
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] accesses: An array of objects that define dataset access for one or more entities.  Structure is documented below.
        :param pulumi.Input[str] dataset_id: The ID of the dataset containing this table.
        :param pulumi.Input[dict] default_encryption_configuration: The default encryption key for all tables in the dataset. Once this property is set,
               all newly-created partitioned tables in the dataset will have encryption key set to
               this value, unless table creation request (or query) overrides the key.  Structure is documented below.
        :param pulumi.Input[float] default_partition_expiration_ms: The default partition expiration for all partitioned tables in
               the dataset, in milliseconds.
        :param pulumi.Input[float] default_table_expiration_ms: The default lifetime of all tables in the dataset, in milliseconds.
               The minimum value is 3600000 milliseconds (one hour).
        :param pulumi.Input[bool] delete_contents_on_destroy: If set to `true`, delete all the tables in the
               dataset when destroying the resource; otherwise,
               destroying the resource will fail if tables are present.
        :param pulumi.Input[str] description: A user-friendly description of the dataset
        :param pulumi.Input[str] friendly_name: A descriptive name for the dataset
        :param pulumi.Input[dict] labels: The labels associated with this dataset. You can use these to
               organize and group your datasets
        :param pulumi.Input[str] location: The geographic location where the dataset should reside.
               See [official docs](https://cloud.google.com/bigquery/docs/dataset-locations).
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **accesses** object supports the following:

          * `domain` (`pulumi.Input[str]`) - A domain to grant access to. Any users signed in with the
            domain specified will be granted the specified access
          * `group_by_email` (`pulumi.Input[str]`) - An email address of a Google Group to grant access to.
          * `role` (`pulumi.Input[str]`) - Describes the rights granted to the user specified by the other
            member of the access object. Primitive, Predefined and custom
            roles are supported. Predefined roles that have equivalent
            primitive roles are swapped by the API to their Primitive
            counterparts. See
            [official docs](https://cloud.google.com/bigquery/docs/access-control).
          * `special_group` (`pulumi.Input[str]`) - A special group to grant access to. Possible values include:
          * `user_by_email` (`pulumi.Input[str]`) - An email address of a user to grant access to. For example:
            fred@example.com
          * `view` (`pulumi.Input[dict]`) - A view from a different dataset to grant access to. Queries
            executed against that view will have read access to tables in
            this dataset. The role field is not required when this field is
            set. If that view is updated by any user, access to the view
            needs to be granted again via an update operation.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this table.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this table.
            * `table_id` (`pulumi.Input[str]`) - The ID of the table. The ID must contain only letters (a-z,
              A-Z), numbers (0-9), or underscores (_). The maximum length
              is 1,024 characters.

        The **default_encryption_configuration** object supports the following:

          * `kms_key_name` (`pulumi.Input[str]`) - Describes the Cloud KMS encryption key that will be used to protect destination
            BigQuery table. The BigQuery Service Account associated with your project requires
            access to this encryption key.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['accesses'] = accesses
            if dataset_id is None:
                raise TypeError("Missing required property 'dataset_id'")
            __props__['dataset_id'] = dataset_id
            __props__['default_encryption_configuration'] = default_encryption_configuration
            __props__['default_partition_expiration_ms'] = default_partition_expiration_ms
            __props__['default_table_expiration_ms'] = default_table_expiration_ms
            __props__['delete_contents_on_destroy'] = delete_contents_on_destroy
            __props__['description'] = description
            __props__['friendly_name'] = friendly_name
            __props__['labels'] = labels
            __props__['location'] = location
            __props__['project'] = project
            __props__['creation_time'] = None
            __props__['etag'] = None
            __props__['last_modified_time'] = None
            __props__['self_link'] = None
        super(Dataset, __self__).__init__(
            'gcp:bigquery/dataset:Dataset',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, accesses=None, creation_time=None, dataset_id=None, default_encryption_configuration=None, default_partition_expiration_ms=None, default_table_expiration_ms=None, delete_contents_on_destroy=None, description=None, etag=None, friendly_name=None, labels=None, last_modified_time=None, location=None, project=None, self_link=None):
        """
        Get an existing Dataset resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] accesses: An array of objects that define dataset access for one or more entities.  Structure is documented below.
        :param pulumi.Input[float] creation_time: The time when this dataset was created, in milliseconds since the epoch.
        :param pulumi.Input[str] dataset_id: The ID of the dataset containing this table.
        :param pulumi.Input[dict] default_encryption_configuration: The default encryption key for all tables in the dataset. Once this property is set,
               all newly-created partitioned tables in the dataset will have encryption key set to
               this value, unless table creation request (or query) overrides the key.  Structure is documented below.
        :param pulumi.Input[float] default_partition_expiration_ms: The default partition expiration for all partitioned tables in
               the dataset, in milliseconds.
        :param pulumi.Input[float] default_table_expiration_ms: The default lifetime of all tables in the dataset, in milliseconds.
               The minimum value is 3600000 milliseconds (one hour).
        :param pulumi.Input[bool] delete_contents_on_destroy: If set to `true`, delete all the tables in the
               dataset when destroying the resource; otherwise,
               destroying the resource will fail if tables are present.
        :param pulumi.Input[str] description: A user-friendly description of the dataset
        :param pulumi.Input[str] etag: A hash of the resource.
        :param pulumi.Input[str] friendly_name: A descriptive name for the dataset
        :param pulumi.Input[dict] labels: The labels associated with this dataset. You can use these to
               organize and group your datasets
        :param pulumi.Input[float] last_modified_time: The date when this dataset or any of its tables was last modified, in milliseconds since the epoch.
        :param pulumi.Input[str] location: The geographic location where the dataset should reside.
               See [official docs](https://cloud.google.com/bigquery/docs/dataset-locations).
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] self_link: The URI of the created resource.

        The **accesses** object supports the following:

          * `domain` (`pulumi.Input[str]`) - A domain to grant access to. Any users signed in with the
            domain specified will be granted the specified access
          * `group_by_email` (`pulumi.Input[str]`) - An email address of a Google Group to grant access to.
          * `role` (`pulumi.Input[str]`) - Describes the rights granted to the user specified by the other
            member of the access object. Primitive, Predefined and custom
            roles are supported. Predefined roles that have equivalent
            primitive roles are swapped by the API to their Primitive
            counterparts. See
            [official docs](https://cloud.google.com/bigquery/docs/access-control).
          * `special_group` (`pulumi.Input[str]`) - A special group to grant access to. Possible values include:
          * `user_by_email` (`pulumi.Input[str]`) - An email address of a user to grant access to. For example:
            fred@example.com
          * `view` (`pulumi.Input[dict]`) - A view from a different dataset to grant access to. Queries
            executed against that view will have read access to tables in
            this dataset. The role field is not required when this field is
            set. If that view is updated by any user, access to the view
            needs to be granted again via an update operation.  Structure is documented below.
            * `dataset_id` (`pulumi.Input[str]`) - The ID of the dataset containing this table.
            * `project_id` (`pulumi.Input[str]`) - The ID of the project containing this table.
            * `table_id` (`pulumi.Input[str]`) - The ID of the table. The ID must contain only letters (a-z,
              A-Z), numbers (0-9), or underscores (_). The maximum length
              is 1,024 characters.

        The **default_encryption_configuration** object supports the following:

          * `kms_key_name` (`pulumi.Input[str]`) - Describes the Cloud KMS encryption key that will be used to protect destination
            BigQuery table. The BigQuery Service Account associated with your project requires
            access to this encryption key.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["accesses"] = accesses
        __props__["creation_time"] = creation_time
        __props__["dataset_id"] = dataset_id
        __props__["default_encryption_configuration"] = default_encryption_configuration
        __props__["default_partition_expiration_ms"] = default_partition_expiration_ms
        __props__["default_table_expiration_ms"] = default_table_expiration_ms
        __props__["delete_contents_on_destroy"] = delete_contents_on_destroy
        __props__["description"] = description
        __props__["etag"] = etag
        __props__["friendly_name"] = friendly_name
        __props__["labels"] = labels
        __props__["last_modified_time"] = last_modified_time
        __props__["location"] = location
        __props__["project"] = project
        __props__["self_link"] = self_link
        return Dataset(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

warnings.warn("gcp.compute.MangedSslCertificate has been deprecated in favor of gcp.compute.ManagedSslCertificate", DeprecationWarning)
class MangedSslCertificate(pulumi.CustomResource):
    certificate_id: pulumi.Output[float]
    """
    The unique identifier for the resource.
    """
    creation_timestamp: pulumi.Output[str]
    """
    Creation timestamp in RFC3339 text format.
    """
    description: pulumi.Output[str]
    """
    An optional description of this resource.
    """
    expire_time: pulumi.Output[str]
    """
    Expire time of the certificate.
    """
    managed: pulumi.Output[dict]
    """
    Properties relevant to a managed certificate. These will be used if the certificate is managed (as indicated by a value
    of 'MANAGED' in 'type').

      * `domains` (`list`)
    """
    name: pulumi.Output[str]
    """
    Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and
    comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression
    '[a-z]([-a-z0-9]*[a-z0-9])?' which means the first character must be a lowercase letter, and all following characters
    must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash. These are in the same
    namespace as the managed SSL certificates.
    """
    project: pulumi.Output[str]
    self_link: pulumi.Output[str]
    subject_alternative_names: pulumi.Output[list]
    """
    Domains associated with the certificate via Subject Alternative Name.
    """
    type: pulumi.Output[str]
    """
    Enum field whose value is always 'MANAGED' - used to signal to the API which type this is. Default value: "MANAGED"
    Possible values: ["MANAGED"]
    """
    warnings.warn("gcp.compute.MangedSslCertificate has been deprecated in favor of gcp.compute.ManagedSslCertificate", DeprecationWarning)
    def __init__(__self__, resource_name, opts=None, certificate_id=None, description=None, managed=None, name=None, project=None, type=None, __props__=None, __name__=None, __opts__=None):
        """
        Create a MangedSslCertificate resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[float] certificate_id: The unique identifier for the resource.
        :param pulumi.Input[str] description: An optional description of this resource.
        :param pulumi.Input[dict] managed: Properties relevant to a managed certificate. These will be used if the certificate is managed (as indicated by a value
               of 'MANAGED' in 'type').
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and
               comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression
               '[a-z]([-a-z0-9]*[a-z0-9])?' which means the first character must be a lowercase letter, and all following characters
               must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash. These are in the same
               namespace as the managed SSL certificates.
        :param pulumi.Input[str] type: Enum field whose value is always 'MANAGED' - used to signal to the API which type this is. Default value: "MANAGED"
               Possible values: ["MANAGED"]

        The **managed** object supports the following:

          * `domains` (`pulumi.Input[list]`)
        """
        pulumi.log.warn("MangedSslCertificate is deprecated: gcp.compute.MangedSslCertificate has been deprecated in favor of gcp.compute.ManagedSslCertificate")
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['certificate_id'] = certificate_id
            __props__['description'] = description
            __props__['managed'] = managed
            __props__['name'] = name
            __props__['project'] = project
            __props__['type'] = type
            __props__['creation_timestamp'] = None
            __props__['expire_time'] = None
            __props__['self_link'] = None
            __props__['subject_alternative_names'] = None
        super(MangedSslCertificate, __self__).__init__(
            'gcp:compute/mangedSslCertificate:MangedSslCertificate',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, certificate_id=None, creation_timestamp=None, description=None, expire_time=None, managed=None, name=None, project=None, self_link=None, subject_alternative_names=None, type=None):
        """
        Get an existing MangedSslCertificate resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[float] certificate_id: The unique identifier for the resource.
        :param pulumi.Input[str] creation_timestamp: Creation timestamp in RFC3339 text format.
        :param pulumi.Input[str] description: An optional description of this resource.
        :param pulumi.Input[str] expire_time: Expire time of the certificate.
        :param pulumi.Input[dict] managed: Properties relevant to a managed certificate. These will be used if the certificate is managed (as indicated by a value
               of 'MANAGED' in 'type').
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and
               comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression
               '[a-z]([-a-z0-9]*[a-z0-9])?' which means the first character must be a lowercase letter, and all following characters
               must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash. These are in the same
               namespace as the managed SSL certificates.
        :param pulumi.Input[list] subject_alternative_names: Domains associated with the certificate via Subject Alternative Name.
        :param pulumi.Input[str] type: Enum field whose value is always 'MANAGED' - used to signal to the API which type this is. Default value: "MANAGED"
               Possible values: ["MANAGED"]

        The **managed** object supports the following:

          * `domains` (`pulumi.Input[list]`)
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["certificate_id"] = certificate_id
        __props__["creation_timestamp"] = creation_timestamp
        __props__["description"] = description
        __props__["expire_time"] = expire_time
        __props__["managed"] = managed
        __props__["name"] = name
        __props__["project"] = project
        __props__["self_link"] = self_link
        __props__["subject_alternative_names"] = subject_alternative_names
        __props__["type"] = type
        return MangedSslCertificate(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Instance(pulumi.CustomResource):
    allow_stopping_for_update: pulumi.Output[bool]
    """
    If true, allows this prvider to stop the instance to update its properties.
    If you try to update a property that requires stopping the instance without setting this field, the update will fail.
    """
    attached_disks: pulumi.Output[list]
    """
    Additional disks to attach to the instance. Can be repeated multiple times for multiple disks. Structure is documented below.

      * `device_name` (`str`) - Name with which the attached disk will be accessible
        under `/dev/disk/by-id/google-*`
      * `diskEncryptionKeyRaw` (`str`) - A 256-bit [customer-supplied encryption key]
        (https://cloud.google.com/compute/docs/disks/customer-supplied-encryption),
        encoded in [RFC 4648 base64](https://tools.ietf.org/html/rfc4648#section-4)
        to encrypt this disk. Only one of `kms_key_self_link` and `disk_encryption_key_raw` may be set.
      * `diskEncryptionKeySha256` (`str`)
      * `kmsKeySelfLink` (`str`) - The self_link of the encryption key that is
        stored in Google Cloud KMS to encrypt this disk. Only one of `kms_key_self_link`
        and `disk_encryption_key_raw` may be set.
      * `mode` (`str`) - Either "READ_ONLY" or "READ_WRITE", defaults to "READ_WRITE"
        If you have a persistent disk with data that you want to share
        between multiple instances, detach it from any read-write instances and
        attach it to one or more instances in read-only mode.
      * `source` (`str`) - The name or self_link of the disk to attach to this instance.
    """
    boot_disk: pulumi.Output[dict]
    """
    The boot disk for the instance.
    Structure is documented below.

      * `autoDelete` (`bool`) - Whether the disk will be auto-deleted when the instance
        is deleted. Defaults to true.
      * `device_name` (`str`) - Name with which the attached disk will be accessible
        under `/dev/disk/by-id/google-*`
      * `diskEncryptionKeyRaw` (`str`) - A 256-bit [customer-supplied encryption key]
        (https://cloud.google.com/compute/docs/disks/customer-supplied-encryption),
        encoded in [RFC 4648 base64](https://tools.ietf.org/html/rfc4648#section-4)
        to encrypt this disk. Only one of `kms_key_self_link` and `disk_encryption_key_raw` may be set.
      * `diskEncryptionKeySha256` (`str`)
      * `initializeParams` (`dict`) - Parameters for a new disk that will be created
        alongside the new instance. Either `initialize_params` or `source` must be set.
        Structure is documented below.
        * `image` (`str`) - The image from which to initialize this disk. This can be
          one of: the image's `self_link`, `projects/{project}/global/images/{image}`,
          `projects/{project}/global/images/family/{family}`, `global/images/{image}`,
          `global/images/family/{family}`, `family/{family}`, `{project}/{family}`,
          `{project}/{image}`, `{family}`, or `{image}`. If referred by family, the
          images names must include the family name. If they don't, use the
          [compute.Image data source](https://www.terraform.io/docs/providers/google/d/datasource_compute_image.html).
          For instance, the image `centos-6-v20180104` includes its family name `centos-6`.
          These images can be referred by family name here.
        * `labels` (`dict`) - A map of key/value label pairs to assign to the instance.
        * `size` (`float`) - The size of the image in gigabytes. If not specified, it
          will inherit the size of its base image.
        * `type` (`str`) - The accelerator type resource to expose to this instance. E.g. `nvidia-tesla-k80`.

      * `kmsKeySelfLink` (`str`) - The self_link of the encryption key that is
        stored in Google Cloud KMS to encrypt this disk. Only one of `kms_key_self_link`
        and `disk_encryption_key_raw` may be set.
      * `mode` (`str`) - Either "READ_ONLY" or "READ_WRITE", defaults to "READ_WRITE"
        If you have a persistent disk with data that you want to share
        between multiple instances, detach it from any read-write instances and
        attach it to one or more instances in read-only mode.
      * `source` (`str`) - The name or self_link of the disk to attach to this instance.
    """
    can_ip_forward: pulumi.Output[bool]
    """
    Whether to allow sending and receiving of
    packets with non-matching source or destination IPs.
    This defaults to false.
    """
    cpu_platform: pulumi.Output[str]
    """
    The CPU platform used by this instance.
    """
    current_status: pulumi.Output[str]
    deletion_protection: pulumi.Output[bool]
    """
    Enable deletion protection on this instance. Defaults to false.
    **Note:** you must disable deletion protection before removing the resource (e.g., via `pulumi destroy`), or the instance cannot be deleted and the provider run will not complete successfully.
    """
    description: pulumi.Output[str]
    """
    A brief description of this resource.
    """
    desired_status: pulumi.Output[str]
    """
    Desired status of the instance. Either
    `"RUNNING"` or `"TERMINATED"`.
    """
    enable_display: pulumi.Output[bool]
    """
    Enable [Virtual Displays](https://cloud.google.com/compute/docs/instances/enable-instance-virtual-display#verify_display_driver) on this instance.
    **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
    """
    guest_accelerators: pulumi.Output[list]
    """
    List of the type and count of accelerator cards attached to the instance. Structure documented below.
    **Note:** GPU accelerators can only be used with `on_host_maintenance` option set to TERMINATE.

      * `count` (`float`) - The number of the guest accelerator cards exposed to this instance.
      * `type` (`str`) - The accelerator type resource to expose to this instance. E.g. `nvidia-tesla-k80`.
    """
    hostname: pulumi.Output[str]
    """
    A custom hostname for the instance. Must be a fully qualified DNS name and RFC-1035-valid.
    Valid format is a series of labels 1-63 characters long matching the regular expression `a-z`, concatenated with periods.
    The entire hostname must not exceed 253 characters. Changing this forces a new resource to be created.
    """
    instance_id: pulumi.Output[str]
    """
    The server-assigned unique identifier of this instance.
    """
    label_fingerprint: pulumi.Output[str]
    """
    The unique fingerprint of the labels.
    """
    labels: pulumi.Output[dict]
    """
    A map of key/value label pairs to assign to the instance.
    """
    machine_type: pulumi.Output[str]
    """
    The machine type to create.
    """
    metadata: pulumi.Output[dict]
    """
    Metadata key/value pairs to make available from
    within the instance. Ssh keys attached in the Cloud Console will be removed.
    Add them to your config in order to keep them attached to your instance.
    """
    metadata_fingerprint: pulumi.Output[str]
    """
    The unique fingerprint of the metadata.
    """
    metadata_startup_script: pulumi.Output[str]
    """
    An alternative to using the
    startup-script metadata key, except this one forces the instance to be
    recreated (thus re-running the script) if it is changed. This replaces the
    startup-script metadata key on the created instance and thus the two
    mechanisms are not allowed to be used simultaneously.  Users are free to use
    either mechanism - the only distinction is that this separate attribute
    willl cause a recreate on modification.  On import, `metadata_startup_script`
    will be set, but `metadata.startup-script` will not - if you choose to use the
    other mechanism, you will see a diff immediately after import, which will cause a
    destroy/recreate operation.  You may want to modify your state file manually
    using `pulumi stack` commands, depending on your use case.
    """
    min_cpu_platform: pulumi.Output[str]
    """
    Specifies a minimum CPU platform for the VM instance. Applicable values are the friendly names of CPU platforms, such as
    `Intel Haswell` or `Intel Skylake`. See the complete list [here](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform).
    **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
    """
    name: pulumi.Output[str]
    """
    A unique name for the resource, required by GCE.
    Changing this forces a new resource to be created.
    """
    network_interfaces: pulumi.Output[list]
    """
    Networks to attach to the instance. This can
    be specified multiple times. Structure is documented below.

      * `accessConfigs` (`list`) - Access configurations, i.e. IPs via which this
        instance can be accessed via the Internet. Omit to ensure that the instance
        is not accessible from the Internet. If omitted, ssh will not
        work unless this provider can send traffic to the instance's network (e.g. via
        tunnel or because it is running on another cloud instance on that network).
        This block can be repeated multiple times. Structure documented below.
        * `natIp` (`str`) - The IP address that will be 1:1 mapped to the instance's
          network ip. If not given, one will be generated.
        * `network_tier` (`str`) - The [networking tier][network-tier] used for configuring this instance.
          This field can take the following values: PREMIUM or STANDARD. If this field is
          not specified, it is assumed to be PREMIUM.
        * `publicPtrDomainName` (`str`) - The DNS domain name for the public PTR record.
          To set this field on an instance, you must be verified as the owner of the domain.
          See [the docs](https://cloud.google.com/compute/docs/instances/create-ptr-record) for how
          to become verified as a domain owner.

      * `aliasIpRanges` (`list`) - An
        array of alias IP ranges for this network interface. Can only be specified for network
        interfaces on subnet-mode networks. Structure documented below.
        * `ip_cidr_range` (`str`) - The IP CIDR range represented by this alias IP range. This IP CIDR range
          must belong to the specified subnetwork and cannot contain IP addresses reserved by
          system or used by other network interfaces. This range may be a single IP address
          (e.g. 10.2.3.4), a netmask (e.g. /24) or a CIDR format string (e.g. 10.1.2.0/24).
        * `subnetworkRangeName` (`str`) - The subnetwork secondary range name specifying
          the secondary range from which to allocate the IP CIDR range for this alias IP
          range. If left unspecified, the primary range of the subnetwork will be used.

      * `name` (`str`) - A unique name for the resource, required by GCE.
        Changing this forces a new resource to be created.
      * `network` (`str`) - The name or self_link of the network to attach this interface to.
        Either `network` or `subnetwork` must be provided.
      * `networkIp` (`str`) - The private IP address to assign to the instance. If
        empty, the address will be automatically assigned.
      * `subnetwork` (`str`) - The name or self_link of the subnetwork to attach this
        interface to. The subnetwork must exist in the same region this instance will be
        created in. Either `network` or `subnetwork` must be provided.
      * `subnetworkProject` (`str`) - The project in which the subnetwork belongs.
        If the `subnetwork` is a self_link, this field is ignored in favor of the project
        defined in the subnetwork self_link. If the `subnetwork` is a name and this
        field is not provided, the provider project is used.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs. If it
    is not provided, the provider project is used.
    """
    resource_policies: pulumi.Output[str]
    """
    -- A list of short names or self_links of resource policies to attach to the instance. Modifying this list will cause the instance to recreate. Currently a max of 1 resource policy is supported.
    """
    scheduling: pulumi.Output[dict]
    """
    The scheduling strategy to use. More details about
    this configuration option are detailed below.

      * `automaticRestart` (`bool`) - Specifies if the instance should be
        restarted if it was terminated by Compute Engine (not a user).
        Defaults to true.
      * `nodeAffinities` (`list`) - Specifies node affinities or anti-affinities
        to determine which sole-tenant nodes your instances and managed instance
        groups will use as host systems. Read more on sole-tenant node creation
        [here](https://cloud.google.com/compute/docs/nodes/create-nodes).
        Structure documented below.
        * `key` (`str`) - The key for the node affinity label.
        * `operator` (`str`) - The operator. Can be `IN` for node-affinities
          or `NOT_IN` for anti-affinities.
        * `values` (`list`)

      * `onHostMaintenance` (`str`) - Describes maintenance behavior for the
        instance. Can be MIGRATE or TERMINATE, for more info, read
        [here](https://cloud.google.com/compute/docs/instances/setting-instance-scheduling-options).
      * `preemptible` (`bool`) - Specifies if the instance is preemptible.
        If this field is set to true, then `automatic_restart` must be
        set to false.  Defaults to false.
    """
    scratch_disks: pulumi.Output[list]
    """
    Scratch disks to attach to the instance. This can be
    specified multiple times for multiple scratch disks. Structure is documented below.

      * `interface` (`str`) - The disk interface to use for attaching this disk; either SCSI or NVME.
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    service_account: pulumi.Output[dict]
    """
    Service account to attach to the instance.
    Structure is documented below.
    **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.

      * `email` (`str`) - The service account e-mail address. If not given, the
        default Google Compute Engine service account is used.
        **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
      * `scopes` (`list`) - A list of service scopes. Both OAuth2 URLs and gcloud
        short names are supported. To allow full access to all Cloud APIs, use the
        `cloud-platform` scope. See a complete list of scopes [here](https://cloud.google.com/sdk/gcloud/reference/alpha/compute/instances/set-scopes#--scopes).
        **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
    """
    shielded_instance_config: pulumi.Output[dict]
    """
    Enable [Shielded VM](https://cloud.google.com/security/shielded-cloud/shielded-vm) on this instance. Shielded VM provides verifiable integrity to prevent against malware and rootkits. Defaults to disabled. Structure is documented below.
    **Note**: `shielded_instance_config` can only be used with boot images with shielded vm support. See the complete list [here](https://cloud.google.com/compute/docs/images#shielded-images).

      * `enableIntegrityMonitoring` (`bool`) - -- Compare the most recent boot measurements to the integrity policy baseline and return a pair of pass/fail results depending on whether they match or not. Defaults to true.
      * `enableSecureBoot` (`bool`) - -- Verify the digital signature of all boot components, and halt the boot process if signature verification fails. Defaults to false.
      * `enableVtpm` (`bool`) - -- Use a virtualized trusted platform module, which is a specialized computer chip you can use to encrypt objects like keys and certificates. Defaults to true.
    """
    tags: pulumi.Output[list]
    """
    A list of tags to attach to the instance.
    """
    tags_fingerprint: pulumi.Output[str]
    """
    The unique fingerprint of the tags.
    """
    zone: pulumi.Output[str]
    """
    The zone that the machine should be created in.
    """
    def __init__(__self__, resource_name, opts=None, allow_stopping_for_update=None, attached_disks=None, boot_disk=None, can_ip_forward=None, deletion_protection=None, description=None, desired_status=None, enable_display=None, guest_accelerators=None, hostname=None, labels=None, machine_type=None, metadata=None, metadata_startup_script=None, min_cpu_platform=None, name=None, network_interfaces=None, project=None, resource_policies=None, scheduling=None, scratch_disks=None, service_account=None, shielded_instance_config=None, tags=None, zone=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages a VM instance resource within GCE. For more information see
        [the official documentation](https://cloud.google.com/compute/docs/instances)
        and
        [API](https://cloud.google.com/compute/docs/reference/latest/instances).


        ## Example Usage



        ```python
        import pulumi
        import pulumi_gcp as gcp

        default = gcp.compute.Instance("default",
            boot_disk={
                "initializeParams": {
                    "image": "debian-cloud/debian-9",
                },
            },
            machine_type="n1-standard-1",
            metadata={
                "foo": "bar",
            },
            metadata_startup_script="echo hi > /test.txt",
            network_interfaces=[{
                "accessConfig": [{}],
                "network": "default",
            }],
            scratch_disks=[{
                "interface": "SCSI",
            }],
            service_account={
                "scopes": [
                    "userinfo-email",
                    "compute-ro",
                    "storage-ro",
                ],
            },
            tags=[
                "foo",
                "bar",
            ],
            zone="us-central1-a")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] allow_stopping_for_update: If true, allows this prvider to stop the instance to update its properties.
               If you try to update a property that requires stopping the instance without setting this field, the update will fail.
        :param pulumi.Input[list] attached_disks: Additional disks to attach to the instance. Can be repeated multiple times for multiple disks. Structure is documented below.
        :param pulumi.Input[dict] boot_disk: The boot disk for the instance.
               Structure is documented below.
        :param pulumi.Input[bool] can_ip_forward: Whether to allow sending and receiving of
               packets with non-matching source or destination IPs.
               This defaults to false.
        :param pulumi.Input[bool] deletion_protection: Enable deletion protection on this instance. Defaults to false.
               **Note:** you must disable deletion protection before removing the resource (e.g., via `pulumi destroy`), or the instance cannot be deleted and the provider run will not complete successfully.
        :param pulumi.Input[str] description: A brief description of this resource.
        :param pulumi.Input[str] desired_status: Desired status of the instance. Either
               `"RUNNING"` or `"TERMINATED"`.
        :param pulumi.Input[bool] enable_display: Enable [Virtual Displays](https://cloud.google.com/compute/docs/instances/enable-instance-virtual-display#verify_display_driver) on this instance.
               **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
        :param pulumi.Input[list] guest_accelerators: List of the type and count of accelerator cards attached to the instance. Structure documented below.
               **Note:** GPU accelerators can only be used with `on_host_maintenance` option set to TERMINATE.
        :param pulumi.Input[str] hostname: A custom hostname for the instance. Must be a fully qualified DNS name and RFC-1035-valid.
               Valid format is a series of labels 1-63 characters long matching the regular expression `a-z`, concatenated with periods.
               The entire hostname must not exceed 253 characters. Changing this forces a new resource to be created.
        :param pulumi.Input[dict] labels: A map of key/value label pairs to assign to the instance.
        :param pulumi.Input[str] machine_type: The machine type to create.
        :param pulumi.Input[dict] metadata: Metadata key/value pairs to make available from
               within the instance. Ssh keys attached in the Cloud Console will be removed.
               Add them to your config in order to keep them attached to your instance.
        :param pulumi.Input[str] metadata_startup_script: An alternative to using the
               startup-script metadata key, except this one forces the instance to be
               recreated (thus re-running the script) if it is changed. This replaces the
               startup-script metadata key on the created instance and thus the two
               mechanisms are not allowed to be used simultaneously.  Users are free to use
               either mechanism - the only distinction is that this separate attribute
               willl cause a recreate on modification.  On import, `metadata_startup_script`
               will be set, but `metadata.startup-script` will not - if you choose to use the
               other mechanism, you will see a diff immediately after import, which will cause a
               destroy/recreate operation.  You may want to modify your state file manually
               using `pulumi stack` commands, depending on your use case.
        :param pulumi.Input[str] min_cpu_platform: Specifies a minimum CPU platform for the VM instance. Applicable values are the friendly names of CPU platforms, such as
               `Intel Haswell` or `Intel Skylake`. See the complete list [here](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform).
               **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
        :param pulumi.Input[str] name: A unique name for the resource, required by GCE.
               Changing this forces a new resource to be created.
        :param pulumi.Input[list] network_interfaces: Networks to attach to the instance. This can
               be specified multiple times. Structure is documented below.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs. If it
               is not provided, the provider project is used.
        :param pulumi.Input[str] resource_policies: -- A list of short names or self_links of resource policies to attach to the instance. Modifying this list will cause the instance to recreate. Currently a max of 1 resource policy is supported.
        :param pulumi.Input[dict] scheduling: The scheduling strategy to use. More details about
               this configuration option are detailed below.
        :param pulumi.Input[list] scratch_disks: Scratch disks to attach to the instance. This can be
               specified multiple times for multiple scratch disks. Structure is documented below.
        :param pulumi.Input[dict] service_account: Service account to attach to the instance.
               Structure is documented below.
               **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
        :param pulumi.Input[dict] shielded_instance_config: Enable [Shielded VM](https://cloud.google.com/security/shielded-cloud/shielded-vm) on this instance. Shielded VM provides verifiable integrity to prevent against malware and rootkits. Defaults to disabled. Structure is documented below.
               **Note**: `shielded_instance_config` can only be used with boot images with shielded vm support. See the complete list [here](https://cloud.google.com/compute/docs/images#shielded-images).
        :param pulumi.Input[list] tags: A list of tags to attach to the instance.
        :param pulumi.Input[str] zone: The zone that the machine should be created in.

        The **attached_disks** object supports the following:

          * `device_name` (`pulumi.Input[str]`) - Name with which the attached disk will be accessible
            under `/dev/disk/by-id/google-*`
          * `diskEncryptionKeyRaw` (`pulumi.Input[str]`) - A 256-bit [customer-supplied encryption key]
            (https://cloud.google.com/compute/docs/disks/customer-supplied-encryption),
            encoded in [RFC 4648 base64](https://tools.ietf.org/html/rfc4648#section-4)
            to encrypt this disk. Only one of `kms_key_self_link` and `disk_encryption_key_raw` may be set.
          * `diskEncryptionKeySha256` (`pulumi.Input[str]`)
          * `kmsKeySelfLink` (`pulumi.Input[str]`) - The self_link of the encryption key that is
            stored in Google Cloud KMS to encrypt this disk. Only one of `kms_key_self_link`
            and `disk_encryption_key_raw` may be set.
          * `mode` (`pulumi.Input[str]`) - Either "READ_ONLY" or "READ_WRITE", defaults to "READ_WRITE"
            If you have a persistent disk with data that you want to share
            between multiple instances, detach it from any read-write instances and
            attach it to one or more instances in read-only mode.
          * `source` (`pulumi.Input[str]`) - The name or self_link of the disk to attach to this instance.

        The **boot_disk** object supports the following:

          * `autoDelete` (`pulumi.Input[bool]`) - Whether the disk will be auto-deleted when the instance
            is deleted. Defaults to true.
          * `device_name` (`pulumi.Input[str]`) - Name with which the attached disk will be accessible
            under `/dev/disk/by-id/google-*`
          * `diskEncryptionKeyRaw` (`pulumi.Input[str]`) - A 256-bit [customer-supplied encryption key]
            (https://cloud.google.com/compute/docs/disks/customer-supplied-encryption),
            encoded in [RFC 4648 base64](https://tools.ietf.org/html/rfc4648#section-4)
            to encrypt this disk. Only one of `kms_key_self_link` and `disk_encryption_key_raw` may be set.
          * `diskEncryptionKeySha256` (`pulumi.Input[str]`)
          * `initializeParams` (`pulumi.Input[dict]`) - Parameters for a new disk that will be created
            alongside the new instance. Either `initialize_params` or `source` must be set.
            Structure is documented below.
            * `image` (`pulumi.Input[str]`) - The image from which to initialize this disk. This can be
              one of: the image's `self_link`, `projects/{project}/global/images/{image}`,
              `projects/{project}/global/images/family/{family}`, `global/images/{image}`,
              `global/images/family/{family}`, `family/{family}`, `{project}/{family}`,
              `{project}/{image}`, `{family}`, or `{image}`. If referred by family, the
              images names must include the family name. If they don't, use the
              [compute.Image data source](https://www.terraform.io/docs/providers/google/d/datasource_compute_image.html).
              For instance, the image `centos-6-v20180104` includes its family name `centos-6`.
              These images can be referred by family name here.
            * `labels` (`pulumi.Input[dict]`) - A map of key/value label pairs to assign to the instance.
            * `size` (`pulumi.Input[float]`) - The size of the image in gigabytes. If not specified, it
              will inherit the size of its base image.
            * `type` (`pulumi.Input[str]`) - The accelerator type resource to expose to this instance. E.g. `nvidia-tesla-k80`.

          * `kmsKeySelfLink` (`pulumi.Input[str]`) - The self_link of the encryption key that is
            stored in Google Cloud KMS to encrypt this disk. Only one of `kms_key_self_link`
            and `disk_encryption_key_raw` may be set.
          * `mode` (`pulumi.Input[str]`) - Either "READ_ONLY" or "READ_WRITE", defaults to "READ_WRITE"
            If you have a persistent disk with data that you want to share
            between multiple instances, detach it from any read-write instances and
            attach it to one or more instances in read-only mode.
          * `source` (`pulumi.Input[str]`) - The name or self_link of the disk to attach to this instance.

        The **guest_accelerators** object supports the following:

          * `count` (`pulumi.Input[float]`) - The number of the guest accelerator cards exposed to this instance.
          * `type` (`pulumi.Input[str]`) - The accelerator type resource to expose to this instance. E.g. `nvidia-tesla-k80`.

        The **network_interfaces** object supports the following:

          * `accessConfigs` (`pulumi.Input[list]`) - Access configurations, i.e. IPs via which this
            instance can be accessed via the Internet. Omit to ensure that the instance
            is not accessible from the Internet. If omitted, ssh will not
            work unless this provider can send traffic to the instance's network (e.g. via
            tunnel or because it is running on another cloud instance on that network).
            This block can be repeated multiple times. Structure documented below.
            * `natIp` (`pulumi.Input[str]`) - The IP address that will be 1:1 mapped to the instance's
              network ip. If not given, one will be generated.
            * `network_tier` (`pulumi.Input[str]`) - The [networking tier][network-tier] used for configuring this instance.
              This field can take the following values: PREMIUM or STANDARD. If this field is
              not specified, it is assumed to be PREMIUM.
            * `publicPtrDomainName` (`pulumi.Input[str]`) - The DNS domain name for the public PTR record.
              To set this field on an instance, you must be verified as the owner of the domain.
              See [the docs](https://cloud.google.com/compute/docs/instances/create-ptr-record) for how
              to become verified as a domain owner.

          * `aliasIpRanges` (`pulumi.Input[list]`) - An
            array of alias IP ranges for this network interface. Can only be specified for network
            interfaces on subnet-mode networks. Structure documented below.
            * `ip_cidr_range` (`pulumi.Input[str]`) - The IP CIDR range represented by this alias IP range. This IP CIDR range
              must belong to the specified subnetwork and cannot contain IP addresses reserved by
              system or used by other network interfaces. This range may be a single IP address
              (e.g. 10.2.3.4), a netmask (e.g. /24) or a CIDR format string (e.g. 10.1.2.0/24).
            * `subnetworkRangeName` (`pulumi.Input[str]`) - The subnetwork secondary range name specifying
              the secondary range from which to allocate the IP CIDR range for this alias IP
              range. If left unspecified, the primary range of the subnetwork will be used.

          * `name` (`pulumi.Input[str]`) - A unique name for the resource, required by GCE.
            Changing this forces a new resource to be created.
          * `network` (`pulumi.Input[str]`) - The name or self_link of the network to attach this interface to.
            Either `network` or `subnetwork` must be provided.
          * `networkIp` (`pulumi.Input[str]`) - The private IP address to assign to the instance. If
            empty, the address will be automatically assigned.
          * `subnetwork` (`pulumi.Input[str]`) - The name or self_link of the subnetwork to attach this
            interface to. The subnetwork must exist in the same region this instance will be
            created in. Either `network` or `subnetwork` must be provided.
          * `subnetworkProject` (`pulumi.Input[str]`) - The project in which the subnetwork belongs.
            If the `subnetwork` is a self_link, this field is ignored in favor of the project
            defined in the subnetwork self_link. If the `subnetwork` is a name and this
            field is not provided, the provider project is used.

        The **scheduling** object supports the following:

          * `automaticRestart` (`pulumi.Input[bool]`) - Specifies if the instance should be
            restarted if it was terminated by Compute Engine (not a user).
            Defaults to true.
          * `nodeAffinities` (`pulumi.Input[list]`) - Specifies node affinities or anti-affinities
            to determine which sole-tenant nodes your instances and managed instance
            groups will use as host systems. Read more on sole-tenant node creation
            [here](https://cloud.google.com/compute/docs/nodes/create-nodes).
            Structure documented below.
            * `key` (`pulumi.Input[str]`) - The key for the node affinity label.
            * `operator` (`pulumi.Input[str]`) - The operator. Can be `IN` for node-affinities
              or `NOT_IN` for anti-affinities.
            * `values` (`pulumi.Input[list]`)

          * `onHostMaintenance` (`pulumi.Input[str]`) - Describes maintenance behavior for the
            instance. Can be MIGRATE or TERMINATE, for more info, read
            [here](https://cloud.google.com/compute/docs/instances/setting-instance-scheduling-options).
          * `preemptible` (`pulumi.Input[bool]`) - Specifies if the instance is preemptible.
            If this field is set to true, then `automatic_restart` must be
            set to false.  Defaults to false.

        The **scratch_disks** object supports the following:

          * `interface` (`pulumi.Input[str]`) - The disk interface to use for attaching this disk; either SCSI or NVME.

        The **service_account** object supports the following:

          * `email` (`pulumi.Input[str]`) - The service account e-mail address. If not given, the
            default Google Compute Engine service account is used.
            **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
          * `scopes` (`pulumi.Input[list]`) - A list of service scopes. Both OAuth2 URLs and gcloud
            short names are supported. To allow full access to all Cloud APIs, use the
            `cloud-platform` scope. See a complete list of scopes [here](https://cloud.google.com/sdk/gcloud/reference/alpha/compute/instances/set-scopes#--scopes).
            **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.

        The **shielded_instance_config** object supports the following:

          * `enableIntegrityMonitoring` (`pulumi.Input[bool]`) - -- Compare the most recent boot measurements to the integrity policy baseline and return a pair of pass/fail results depending on whether they match or not. Defaults to true.
          * `enableSecureBoot` (`pulumi.Input[bool]`) - -- Verify the digital signature of all boot components, and halt the boot process if signature verification fails. Defaults to false.
          * `enableVtpm` (`pulumi.Input[bool]`) - -- Use a virtualized trusted platform module, which is a specialized computer chip you can use to encrypt objects like keys and certificates. Defaults to true.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['allow_stopping_for_update'] = allow_stopping_for_update
            __props__['attached_disks'] = attached_disks
            if boot_disk is None:
                raise TypeError("Missing required property 'boot_disk'")
            __props__['boot_disk'] = boot_disk
            __props__['can_ip_forward'] = can_ip_forward
            __props__['deletion_protection'] = deletion_protection
            __props__['description'] = description
            __props__['desired_status'] = desired_status
            __props__['enable_display'] = enable_display
            __props__['guest_accelerators'] = guest_accelerators
            __props__['hostname'] = hostname
            __props__['labels'] = labels
            if machine_type is None:
                raise TypeError("Missing required property 'machine_type'")
            __props__['machine_type'] = machine_type
            __props__['metadata'] = metadata
            __props__['metadata_startup_script'] = metadata_startup_script
            __props__['min_cpu_platform'] = min_cpu_platform
            __props__['name'] = name
            if network_interfaces is None:
                raise TypeError("Missing required property 'network_interfaces'")
            __props__['network_interfaces'] = network_interfaces
            __props__['project'] = project
            __props__['resource_policies'] = resource_policies
            __props__['scheduling'] = scheduling
            __props__['scratch_disks'] = scratch_disks
            __props__['service_account'] = service_account
            __props__['shielded_instance_config'] = shielded_instance_config
            __props__['tags'] = tags
            __props__['zone'] = zone
            __props__['cpu_platform'] = None
            __props__['current_status'] = None
            __props__['instance_id'] = None
            __props__['label_fingerprint'] = None
            __props__['metadata_fingerprint'] = None
            __props__['self_link'] = None
            __props__['tags_fingerprint'] = None
        super(Instance, __self__).__init__(
            'gcp:compute/instance:Instance',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, allow_stopping_for_update=None, attached_disks=None, boot_disk=None, can_ip_forward=None, cpu_platform=None, current_status=None, deletion_protection=None, description=None, desired_status=None, enable_display=None, guest_accelerators=None, hostname=None, instance_id=None, label_fingerprint=None, labels=None, machine_type=None, metadata=None, metadata_fingerprint=None, metadata_startup_script=None, min_cpu_platform=None, name=None, network_interfaces=None, project=None, resource_policies=None, scheduling=None, scratch_disks=None, self_link=None, service_account=None, shielded_instance_config=None, tags=None, tags_fingerprint=None, zone=None):
        """
        Get an existing Instance resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] allow_stopping_for_update: If true, allows this prvider to stop the instance to update its properties.
               If you try to update a property that requires stopping the instance without setting this field, the update will fail.
        :param pulumi.Input[list] attached_disks: Additional disks to attach to the instance. Can be repeated multiple times for multiple disks. Structure is documented below.
        :param pulumi.Input[dict] boot_disk: The boot disk for the instance.
               Structure is documented below.
        :param pulumi.Input[bool] can_ip_forward: Whether to allow sending and receiving of
               packets with non-matching source or destination IPs.
               This defaults to false.
        :param pulumi.Input[str] cpu_platform: The CPU platform used by this instance.
        :param pulumi.Input[bool] deletion_protection: Enable deletion protection on this instance. Defaults to false.
               **Note:** you must disable deletion protection before removing the resource (e.g., via `pulumi destroy`), or the instance cannot be deleted and the provider run will not complete successfully.
        :param pulumi.Input[str] description: A brief description of this resource.
        :param pulumi.Input[str] desired_status: Desired status of the instance. Either
               `"RUNNING"` or `"TERMINATED"`.
        :param pulumi.Input[bool] enable_display: Enable [Virtual Displays](https://cloud.google.com/compute/docs/instances/enable-instance-virtual-display#verify_display_driver) on this instance.
               **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
        :param pulumi.Input[list] guest_accelerators: List of the type and count of accelerator cards attached to the instance. Structure documented below.
               **Note:** GPU accelerators can only be used with `on_host_maintenance` option set to TERMINATE.
        :param pulumi.Input[str] hostname: A custom hostname for the instance. Must be a fully qualified DNS name and RFC-1035-valid.
               Valid format is a series of labels 1-63 characters long matching the regular expression `a-z`, concatenated with periods.
               The entire hostname must not exceed 253 characters. Changing this forces a new resource to be created.
        :param pulumi.Input[str] instance_id: The server-assigned unique identifier of this instance.
        :param pulumi.Input[str] label_fingerprint: The unique fingerprint of the labels.
        :param pulumi.Input[dict] labels: A map of key/value label pairs to assign to the instance.
        :param pulumi.Input[str] machine_type: The machine type to create.
        :param pulumi.Input[dict] metadata: Metadata key/value pairs to make available from
               within the instance. Ssh keys attached in the Cloud Console will be removed.
               Add them to your config in order to keep them attached to your instance.
        :param pulumi.Input[str] metadata_fingerprint: The unique fingerprint of the metadata.
        :param pulumi.Input[str] metadata_startup_script: An alternative to using the
               startup-script metadata key, except this one forces the instance to be
               recreated (thus re-running the script) if it is changed. This replaces the
               startup-script metadata key on the created instance and thus the two
               mechanisms are not allowed to be used simultaneously.  Users are free to use
               either mechanism - the only distinction is that this separate attribute
               willl cause a recreate on modification.  On import, `metadata_startup_script`
               will be set, but `metadata.startup-script` will not - if you choose to use the
               other mechanism, you will see a diff immediately after import, which will cause a
               destroy/recreate operation.  You may want to modify your state file manually
               using `pulumi stack` commands, depending on your use case.
        :param pulumi.Input[str] min_cpu_platform: Specifies a minimum CPU platform for the VM instance. Applicable values are the friendly names of CPU platforms, such as
               `Intel Haswell` or `Intel Skylake`. See the complete list [here](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform).
               **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
        :param pulumi.Input[str] name: A unique name for the resource, required by GCE.
               Changing this forces a new resource to be created.
        :param pulumi.Input[list] network_interfaces: Networks to attach to the instance. This can
               be specified multiple times. Structure is documented below.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs. If it
               is not provided, the provider project is used.
        :param pulumi.Input[str] resource_policies: -- A list of short names or self_links of resource policies to attach to the instance. Modifying this list will cause the instance to recreate. Currently a max of 1 resource policy is supported.
        :param pulumi.Input[dict] scheduling: The scheduling strategy to use. More details about
               this configuration option are detailed below.
        :param pulumi.Input[list] scratch_disks: Scratch disks to attach to the instance. This can be
               specified multiple times for multiple scratch disks. Structure is documented below.
        :param pulumi.Input[str] self_link: The URI of the created resource.
        :param pulumi.Input[dict] service_account: Service account to attach to the instance.
               Structure is documented below.
               **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
        :param pulumi.Input[dict] shielded_instance_config: Enable [Shielded VM](https://cloud.google.com/security/shielded-cloud/shielded-vm) on this instance. Shielded VM provides verifiable integrity to prevent against malware and rootkits. Defaults to disabled. Structure is documented below.
               **Note**: `shielded_instance_config` can only be used with boot images with shielded vm support. See the complete list [here](https://cloud.google.com/compute/docs/images#shielded-images).
        :param pulumi.Input[list] tags: A list of tags to attach to the instance.
        :param pulumi.Input[str] tags_fingerprint: The unique fingerprint of the tags.
        :param pulumi.Input[str] zone: The zone that the machine should be created in.

        The **attached_disks** object supports the following:

          * `device_name` (`pulumi.Input[str]`) - Name with which the attached disk will be accessible
            under `/dev/disk/by-id/google-*`
          * `diskEncryptionKeyRaw` (`pulumi.Input[str]`) - A 256-bit [customer-supplied encryption key]
            (https://cloud.google.com/compute/docs/disks/customer-supplied-encryption),
            encoded in [RFC 4648 base64](https://tools.ietf.org/html/rfc4648#section-4)
            to encrypt this disk. Only one of `kms_key_self_link` and `disk_encryption_key_raw` may be set.
          * `diskEncryptionKeySha256` (`pulumi.Input[str]`)
          * `kmsKeySelfLink` (`pulumi.Input[str]`) - The self_link of the encryption key that is
            stored in Google Cloud KMS to encrypt this disk. Only one of `kms_key_self_link`
            and `disk_encryption_key_raw` may be set.
          * `mode` (`pulumi.Input[str]`) - Either "READ_ONLY" or "READ_WRITE", defaults to "READ_WRITE"
            If you have a persistent disk with data that you want to share
            between multiple instances, detach it from any read-write instances and
            attach it to one or more instances in read-only mode.
          * `source` (`pulumi.Input[str]`) - The name or self_link of the disk to attach to this instance.

        The **boot_disk** object supports the following:

          * `autoDelete` (`pulumi.Input[bool]`) - Whether the disk will be auto-deleted when the instance
            is deleted. Defaults to true.
          * `device_name` (`pulumi.Input[str]`) - Name with which the attached disk will be accessible
            under `/dev/disk/by-id/google-*`
          * `diskEncryptionKeyRaw` (`pulumi.Input[str]`) - A 256-bit [customer-supplied encryption key]
            (https://cloud.google.com/compute/docs/disks/customer-supplied-encryption),
            encoded in [RFC 4648 base64](https://tools.ietf.org/html/rfc4648#section-4)
            to encrypt this disk. Only one of `kms_key_self_link` and `disk_encryption_key_raw` may be set.
          * `diskEncryptionKeySha256` (`pulumi.Input[str]`)
          * `initializeParams` (`pulumi.Input[dict]`) - Parameters for a new disk that will be created
            alongside the new instance. Either `initialize_params` or `source` must be set.
            Structure is documented below.
            * `image` (`pulumi.Input[str]`) - The image from which to initialize this disk. This can be
              one of: the image's `self_link`, `projects/{project}/global/images/{image}`,
              `projects/{project}/global/images/family/{family}`, `global/images/{image}`,
              `global/images/family/{family}`, `family/{family}`, `{project}/{family}`,
              `{project}/{image}`, `{family}`, or `{image}`. If referred by family, the
              images names must include the family name. If they don't, use the
              [compute.Image data source](https://www.terraform.io/docs/providers/google/d/datasource_compute_image.html).
              For instance, the image `centos-6-v20180104` includes its family name `centos-6`.
              These images can be referred by family name here.
            * `labels` (`pulumi.Input[dict]`) - A map of key/value label pairs to assign to the instance.
            * `size` (`pulumi.Input[float]`) - The size of the image in gigabytes. If not specified, it
              will inherit the size of its base image.
            * `type` (`pulumi.Input[str]`) - The accelerator type resource to expose to this instance. E.g. `nvidia-tesla-k80`.

          * `kmsKeySelfLink` (`pulumi.Input[str]`) - The self_link of the encryption key that is
            stored in Google Cloud KMS to encrypt this disk. Only one of `kms_key_self_link`
            and `disk_encryption_key_raw` may be set.
          * `mode` (`pulumi.Input[str]`) - Either "READ_ONLY" or "READ_WRITE", defaults to "READ_WRITE"
            If you have a persistent disk with data that you want to share
            between multiple instances, detach it from any read-write instances and
            attach it to one or more instances in read-only mode.
          * `source` (`pulumi.Input[str]`) - The name or self_link of the disk to attach to this instance.

        The **guest_accelerators** object supports the following:

          * `count` (`pulumi.Input[float]`) - The number of the guest accelerator cards exposed to this instance.
          * `type` (`pulumi.Input[str]`) - The accelerator type resource to expose to this instance. E.g. `nvidia-tesla-k80`.

        The **network_interfaces** object supports the following:

          * `accessConfigs` (`pulumi.Input[list]`) - Access configurations, i.e. IPs via which this
            instance can be accessed via the Internet. Omit to ensure that the instance
            is not accessible from the Internet. If omitted, ssh will not
            work unless this provider can send traffic to the instance's network (e.g. via
            tunnel or because it is running on another cloud instance on that network).
            This block can be repeated multiple times. Structure documented below.
            * `natIp` (`pulumi.Input[str]`) - The IP address that will be 1:1 mapped to the instance's
              network ip. If not given, one will be generated.
            * `network_tier` (`pulumi.Input[str]`) - The [networking tier][network-tier] used for configuring this instance.
              This field can take the following values: PREMIUM or STANDARD. If this field is
              not specified, it is assumed to be PREMIUM.
            * `publicPtrDomainName` (`pulumi.Input[str]`) - The DNS domain name for the public PTR record.
              To set this field on an instance, you must be verified as the owner of the domain.
              See [the docs](https://cloud.google.com/compute/docs/instances/create-ptr-record) for how
              to become verified as a domain owner.

          * `aliasIpRanges` (`pulumi.Input[list]`) - An
            array of alias IP ranges for this network interface. Can only be specified for network
            interfaces on subnet-mode networks. Structure documented below.
            * `ip_cidr_range` (`pulumi.Input[str]`) - The IP CIDR range represented by this alias IP range. This IP CIDR range
              must belong to the specified subnetwork and cannot contain IP addresses reserved by
              system or used by other network interfaces. This range may be a single IP address
              (e.g. 10.2.3.4), a netmask (e.g. /24) or a CIDR format string (e.g. 10.1.2.0/24).
            * `subnetworkRangeName` (`pulumi.Input[str]`) - The subnetwork secondary range name specifying
              the secondary range from which to allocate the IP CIDR range for this alias IP
              range. If left unspecified, the primary range of the subnetwork will be used.

          * `name` (`pulumi.Input[str]`) - A unique name for the resource, required by GCE.
            Changing this forces a new resource to be created.
          * `network` (`pulumi.Input[str]`) - The name or self_link of the network to attach this interface to.
            Either `network` or `subnetwork` must be provided.
          * `networkIp` (`pulumi.Input[str]`) - The private IP address to assign to the instance. If
            empty, the address will be automatically assigned.
          * `subnetwork` (`pulumi.Input[str]`) - The name or self_link of the subnetwork to attach this
            interface to. The subnetwork must exist in the same region this instance will be
            created in. Either `network` or `subnetwork` must be provided.
          * `subnetworkProject` (`pulumi.Input[str]`) - The project in which the subnetwork belongs.
            If the `subnetwork` is a self_link, this field is ignored in favor of the project
            defined in the subnetwork self_link. If the `subnetwork` is a name and this
            field is not provided, the provider project is used.

        The **scheduling** object supports the following:

          * `automaticRestart` (`pulumi.Input[bool]`) - Specifies if the instance should be
            restarted if it was terminated by Compute Engine (not a user).
            Defaults to true.
          * `nodeAffinities` (`pulumi.Input[list]`) - Specifies node affinities or anti-affinities
            to determine which sole-tenant nodes your instances and managed instance
            groups will use as host systems. Read more on sole-tenant node creation
            [here](https://cloud.google.com/compute/docs/nodes/create-nodes).
            Structure documented below.
            * `key` (`pulumi.Input[str]`) - The key for the node affinity label.
            * `operator` (`pulumi.Input[str]`) - The operator. Can be `IN` for node-affinities
              or `NOT_IN` for anti-affinities.
            * `values` (`pulumi.Input[list]`)

          * `onHostMaintenance` (`pulumi.Input[str]`) - Describes maintenance behavior for the
            instance. Can be MIGRATE or TERMINATE, for more info, read
            [here](https://cloud.google.com/compute/docs/instances/setting-instance-scheduling-options).
          * `preemptible` (`pulumi.Input[bool]`) - Specifies if the instance is preemptible.
            If this field is set to true, then `automatic_restart` must be
            set to false.  Defaults to false.

        The **scratch_disks** object supports the following:

          * `interface` (`pulumi.Input[str]`) - The disk interface to use for attaching this disk; either SCSI or NVME.

        The **service_account** object supports the following:

          * `email` (`pulumi.Input[str]`) - The service account e-mail address. If not given, the
            default Google Compute Engine service account is used.
            **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.
          * `scopes` (`pulumi.Input[list]`) - A list of service scopes. Both OAuth2 URLs and gcloud
            short names are supported. To allow full access to all Cloud APIs, use the
            `cloud-platform` scope. See a complete list of scopes [here](https://cloud.google.com/sdk/gcloud/reference/alpha/compute/instances/set-scopes#--scopes).
            **Note**: `allow_stopping_for_update` must be set to true or your instance must have a `desired_status` of `TERMINATED` in order to update this field.

        The **shielded_instance_config** object supports the following:

          * `enableIntegrityMonitoring` (`pulumi.Input[bool]`) - -- Compare the most recent boot measurements to the integrity policy baseline and return a pair of pass/fail results depending on whether they match or not. Defaults to true.
          * `enableSecureBoot` (`pulumi.Input[bool]`) - -- Verify the digital signature of all boot components, and halt the boot process if signature verification fails. Defaults to false.
          * `enableVtpm` (`pulumi.Input[bool]`) - -- Use a virtualized trusted platform module, which is a specialized computer chip you can use to encrypt objects like keys and certificates. Defaults to true.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["allow_stopping_for_update"] = allow_stopping_for_update
        __props__["attached_disks"] = attached_disks
        __props__["boot_disk"] = boot_disk
        __props__["can_ip_forward"] = can_ip_forward
        __props__["cpu_platform"] = cpu_platform
        __props__["current_status"] = current_status
        __props__["deletion_protection"] = deletion_protection
        __props__["description"] = description
        __props__["desired_status"] = desired_status
        __props__["enable_display"] = enable_display
        __props__["guest_accelerators"] = guest_accelerators
        __props__["hostname"] = hostname
        __props__["instance_id"] = instance_id
        __props__["label_fingerprint"] = label_fingerprint
        __props__["labels"] = labels
        __props__["machine_type"] = machine_type
        __props__["metadata"] = metadata
        __props__["metadata_fingerprint"] = metadata_fingerprint
        __props__["metadata_startup_script"] = metadata_startup_script
        __props__["min_cpu_platform"] = min_cpu_platform
        __props__["name"] = name
        __props__["network_interfaces"] = network_interfaces
        __props__["project"] = project
        __props__["resource_policies"] = resource_policies
        __props__["scheduling"] = scheduling
        __props__["scratch_disks"] = scratch_disks
        __props__["self_link"] = self_link
        __props__["service_account"] = service_account
        __props__["shielded_instance_config"] = shielded_instance_config
        __props__["tags"] = tags
        __props__["tags_fingerprint"] = tags_fingerprint
        __props__["zone"] = zone
        return Instance(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Slo(pulumi.CustomResource):
    basic_sli: pulumi.Output[dict]
    """
    Basic Service-Level Indicator (SLI) on a well-known service type.
    Performance will be computed on the basis of pre-defined metrics.
    SLIs are used to measure and calculate the quality of the Service's
    performance with respect to a single aspect of service quality.
    Exactly one of the following must be set:
    `basic_sli`, `request_based_sli`, `windows_based_sli`  Structure is documented below.

      * `latency` (`dict`) - Parameters for a latency threshold SLI.  Structure is documented below.
        * `threshold` (`str`) - A duration string, e.g. 10s.
          Good service is defined to be the count of requests made to
          this service that return in no more than threshold.

      * `locations` (`list`) - An optional set of locations to which this SLI is relevant.
        Telemetry from other locations will not be used to calculate
        performance for this SLI. If omitted, this SLI applies to all
        locations in which the Service has activity. For service types
        that don't support breaking down by location, setting this
        field will result in an error.
      * `methods` (`list`) - An optional set of RPCs to which this SLI is relevant.
        Telemetry from other methods will not be used to calculate
        performance for this SLI. If omitted, this SLI applies to all
        the Service's methods. For service types that don't support
        breaking down by method, setting this field will result in an
        error.
      * `versions` (`list`) - The set of API versions to which this SLI is relevant.
        Telemetry from other API versions will not be used to
        calculate performance for this SLI. If omitted,
        this SLI applies to all API versions. For service types
        that don't support breaking down by version, setting this
        field will result in an error.
    """
    calendar_period: pulumi.Output[str]
    """
    A calendar period, semantically "since the start of the current
    <calendarPeriod>".
    """
    display_name: pulumi.Output[str]
    """
    Name used for UI elements listing this SLO.
    """
    goal: pulumi.Output[float]
    """
    The fraction of service that must be good in order for this objective
    to be met. 0 < goal <= 0.999
    """
    name: pulumi.Output[str]
    """
    The full resource name for this service. The syntax is:
    projects/[PROJECT_ID_OR_NUMBER]/services/[SERVICE_ID]/serviceLevelObjectives/[SLO_NAME]
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    request_based_sli: pulumi.Output[dict]
    """
    A request-based SLI defines a SLI for which atomic units of
    service are counted directly.
    A SLI describes a good service.
    It is used to measure and calculate the quality of the Service's
    performance with respect to a single aspect of service quality.
    Exactly one of the following must be set:
    `basic_sli`, `request_based_sli`, `windows_based_sli`  Structure is documented below.

      * `distributionCut` (`dict`) - Used when good_service is defined by a count of values aggregated in a
        Distribution that fall into a good range. The total_service is the
        total count of all values aggregated in the Distribution.
        Defines a distribution TimeSeries filter and thresholds used for
        measuring good service and total service.  Structure is documented below.
        * `distributionFilter` (`str`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
          aggregating values to quantify the good service provided.
          Must have ValueType = DISTRIBUTION and
          MetricKind = DELTA or MetricKind = CUMULATIVE.
        * `range` (`dict`) - Range of numerical values. The computed good_service
          will be the count of values x in the Distribution such
          that range.min <= x < range.max. inclusive of min and
          exclusive of max. Open ranges can be defined by setting
          just one of min or max. Summed value `X` should satisfy
          `range.min <= X < range.max` for a good window.  Structure is documented below.
          * `max` (`float`) - max value for the range (inclusive). If not given,
            will be set to "infinity", defining an open range
            ">= range.min"
          * `min` (`float`) - Min value for the range (inclusive). If not given,
            will be set to "-infinity", defining an open range
            "< range.max"

      * `goodTotalRatio` (`dict`) - A means to compute a ratio of `good_service` to `total_service`.
        Defines computing this ratio with two TimeSeries [monitoring filters](https://cloud.google.com/monitoring/api/v3/filters)
        Must specify exactly two of good, bad, and total service filters.
        The relationship good_service + bad_service = total_service
        will be assumed.  Structure is documented below.
        * `badServiceFilter` (`str`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
          quantifying bad service provided, either demanded service that
          was not provided or demanded service that was of inadequate
          quality. Exactly two of
          good, bad, or total service filter must be defined (where
          good + bad = total is assumed)
          Must have ValueType = DOUBLE or ValueType = INT64 and
          must have MetricKind = DELTA or MetricKind = CUMULATIVE.
        * `goodServiceFilter` (`str`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
          quantifying good service provided. Exactly two of
          good, bad, or total service filter must be defined (where
          good + bad = total is assumed)
          Must have ValueType = DOUBLE or ValueType = INT64 and
          must have MetricKind = DELTA or MetricKind = CUMULATIVE.
        * `totalServiceFilter` (`str`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
          quantifying total demanded service. Exactly two of
          good, bad, or total service filter must be defined (where
          good + bad = total is assumed)
          Must have ValueType = DOUBLE or ValueType = INT64 and
          must have MetricKind = DELTA or MetricKind = CUMULATIVE.
    """
    rolling_period_days: pulumi.Output[float]
    """
    A rolling time period, semantically "in the past X days".
    Must be between 1 to 30 days, inclusive.
    """
    service: pulumi.Output[str]
    """
    ID of the service to which this SLO belongs.
    """
    slo_id: pulumi.Output[str]
    """
    The id to use for this ServiceLevelObjective. If omitted, an id will be generated instead.
    """
    windows_based_sli: pulumi.Output[dict]
    """
    A windows-based SLI defines the criteria for time windows.
    good_service is defined based off the count of these time windows
    for which the provided service was of good quality.
    A SLI describes a good service. It is used to measure and calculate
    the quality of the Service's performance with respect to a single
    aspect of service quality.
    Exactly one of the following must be set:
    `basic_sli`, `request_based_sli`, `windows_based_sli`  Structure is documented below.

      * `goodBadMetricFilter` (`str`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
        with ValueType = BOOL. The window is good if any true values
        appear in the window. One of `good_bad_metric_filter`,
        `good_total_ratio_threshold`, `metric_mean_in_range`,
        `metric_sum_in_range` must be set for `windows_based_sli`.
      * `goodTotalRatioThreshold` (`dict`) - Criterion that describes a window as good if its performance is
        high enough. One of `good_bad_metric_filter`,
        `good_total_ratio_threshold`, `metric_mean_in_range`,
        `metric_sum_in_range` must be set for `windows_based_sli`.  Structure is documented below.
        * `basicSliPerformance` (`dict`) - Basic SLI to evaluate to judge window quality.  Structure is documented below.
          * `latency` (`dict`) - Parameters for a latency threshold SLI.  Structure is documented below.
            * `threshold` (`str`) - A duration string, e.g. 10s.
              Good service is defined to be the count of requests made to
              this service that return in no more than threshold.

          * `locations` (`list`) - An optional set of locations to which this SLI is relevant.
            Telemetry from other locations will not be used to calculate
            performance for this SLI. If omitted, this SLI applies to all
            locations in which the Service has activity. For service types
            that don't support breaking down by location, setting this
            field will result in an error.
          * `methods` (`list`) - An optional set of RPCs to which this SLI is relevant.
            Telemetry from other methods will not be used to calculate
            performance for this SLI. If omitted, this SLI applies to all
            the Service's methods. For service types that don't support
            breaking down by method, setting this field will result in an
            error.
          * `versions` (`list`) - The set of API versions to which this SLI is relevant.
            Telemetry from other API versions will not be used to
            calculate performance for this SLI. If omitted,
            this SLI applies to all API versions. For service types
            that don't support breaking down by version, setting this
            field will result in an error.

        * `performance` (`dict`) - Request-based SLI to evaluate to judge window quality.  Structure is documented below.
          * `distributionCut` (`dict`) - Used when good_service is defined by a count of values aggregated in a
            Distribution that fall into a good range. The total_service is the
            total count of all values aggregated in the Distribution.
            Defines a distribution TimeSeries filter and thresholds used for
            measuring good service and total service.  Structure is documented below.
            * `distributionFilter` (`str`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              aggregating values to quantify the good service provided.
              Must have ValueType = DISTRIBUTION and
              MetricKind = DELTA or MetricKind = CUMULATIVE.
            * `range` (`dict`) - Range of numerical values. The computed good_service
              will be the count of values x in the Distribution such
              that range.min <= x < range.max. inclusive of min and
              exclusive of max. Open ranges can be defined by setting
              just one of min or max. Summed value `X` should satisfy
              `range.min <= X < range.max` for a good window.  Structure is documented below.
              * `max` (`float`) - max value for the range (inclusive). If not given,
                will be set to "infinity", defining an open range
                ">= range.min"
              * `min` (`float`) - Min value for the range (inclusive). If not given,
                will be set to "-infinity", defining an open range
                "< range.max"

          * `goodTotalRatio` (`dict`) - A means to compute a ratio of `good_service` to `total_service`.
            Defines computing this ratio with two TimeSeries [monitoring filters](https://cloud.google.com/monitoring/api/v3/filters)
            Must specify exactly two of good, bad, and total service filters.
            The relationship good_service + bad_service = total_service
            will be assumed.  Structure is documented below.
            * `badServiceFilter` (`str`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              quantifying bad service provided, either demanded service that
              was not provided or demanded service that was of inadequate
              quality. Exactly two of
              good, bad, or total service filter must be defined (where
              good + bad = total is assumed)
              Must have ValueType = DOUBLE or ValueType = INT64 and
              must have MetricKind = DELTA or MetricKind = CUMULATIVE.
            * `goodServiceFilter` (`str`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              quantifying good service provided. Exactly two of
              good, bad, or total service filter must be defined (where
              good + bad = total is assumed)
              Must have ValueType = DOUBLE or ValueType = INT64 and
              must have MetricKind = DELTA or MetricKind = CUMULATIVE.
            * `totalServiceFilter` (`str`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              quantifying total demanded service. Exactly two of
              good, bad, or total service filter must be defined (where
              good + bad = total is assumed)
              Must have ValueType = DOUBLE or ValueType = INT64 and
              must have MetricKind = DELTA or MetricKind = CUMULATIVE.

        * `threshold` (`float`) - A duration string, e.g. 10s.
          Good service is defined to be the count of requests made to
          this service that return in no more than threshold.

      * `metricMeanInRange` (`dict`) - Criterion that describes a window as good if the metric's value
        is in a good range, *averaged* across returned streams.
        One of `good_bad_metric_filter`,
        `good_total_ratio_threshold`, `metric_mean_in_range`,
        `metric_sum_in_range` must be set for `windows_based_sli`.
        Average value X of `time_series` should satisfy
        `range.min <= X < range.max` for a good window.  Structure is documented below.
        * `range` (`dict`) - Range of numerical values. The computed good_service
          will be the count of values x in the Distribution such
          that range.min <= x < range.max. inclusive of min and
          exclusive of max. Open ranges can be defined by setting
          just one of min or max. Summed value `X` should satisfy
          `range.min <= X < range.max` for a good window.  Structure is documented below.
          * `max` (`float`) - max value for the range (inclusive). If not given,
            will be set to "infinity", defining an open range
            ">= range.min"
          * `min` (`float`) - Min value for the range (inclusive). If not given,
            will be set to "-infinity", defining an open range
            "< range.max"

        * `timeSeries` (`str`) - A [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
          specifying the TimeSeries to use for evaluating window
          quality. The provided TimeSeries must have
          ValueType = INT64 or ValueType = DOUBLE and
          MetricKind = GAUGE.
          Summed value `X` should satisfy
          `range.min <= X < range.max` for a good window.

      * `metricSumInRange` (`dict`) - Criterion that describes a window as good if the metric's value
        is in a good range, *summed* across returned streams.
        Summed value `X` of `time_series` should satisfy
        `range.min <= X < range.max` for a good window.
        One of `good_bad_metric_filter`,
        `good_total_ratio_threshold`, `metric_mean_in_range`,
        `metric_sum_in_range` must be set for `windows_based_sli`.  Structure is documented below.
        * `range` (`dict`) - Range of numerical values. The computed good_service
          will be the count of values x in the Distribution such
          that range.min <= x < range.max. inclusive of min and
          exclusive of max. Open ranges can be defined by setting
          just one of min or max. Summed value `X` should satisfy
          `range.min <= X < range.max` for a good window.  Structure is documented below.
          * `max` (`float`) - max value for the range (inclusive). If not given,
            will be set to "infinity", defining an open range
            ">= range.min"
          * `min` (`float`) - Min value for the range (inclusive). If not given,
            will be set to "-infinity", defining an open range
            "< range.max"

        * `timeSeries` (`str`) - A [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
          specifying the TimeSeries to use for evaluating window
          quality. The provided TimeSeries must have
          ValueType = INT64 or ValueType = DOUBLE and
          MetricKind = GAUGE.
          Summed value `X` should satisfy
          `range.min <= X < range.max` for a good window.

      * `windowPeriod` (`str`) - Duration over which window quality is evaluated, given as a
        duration string "{X}s" representing X seconds. Must be an
        integer fraction of a day and at least 60s.
    """
    def __init__(__self__, resource_name, opts=None, basic_sli=None, calendar_period=None, display_name=None, goal=None, project=None, request_based_sli=None, rolling_period_days=None, service=None, slo_id=None, windows_based_sli=None, __props__=None, __name__=None, __opts__=None):
        """
        A Service-Level Objective (SLO) describes the level of desired good
        service. It consists of a service-level indicator (SLI), a performance
        goal, and a period over which the objective is to be evaluated against
        that goal. The SLO can use SLIs defined in a number of different manners.
        Typical SLOs might include "99% of requests in each rolling week have
        latency below 200 milliseconds" or "99.5% of requests in each calendar
        month return successfully."


        To get more information about Slo, see:

        * [API documentation](https://cloud.google.com/monitoring/api/ref_v3/rest/v3/services.serviceLevelObjectives)
        * How-to Guides
            * [Service Monitoring](https://cloud.google.com/monitoring/service-monitoring)
            * [Monitoring API Documentation](https://cloud.google.com/monitoring/api/v3/)

        ## Example Usage

        ### Monitoring Slo Appengine

        ```python
        import pulumi
        import pulumi_gcp as gcp

        default = gcp.monitoring.get_app_engine_service(module_id="default")
        appeng_slo = gcp.monitoring.Slo("appengSlo",
            service=default.service_id,
            slo_id="ae-slo",
            display_name="Test SLO for App Engine",
            goal=0.9,
            calendar_period="DAY",
            basic_sli={
                "latency": {
                    "threshold": "1s",
                },
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] basic_sli: Basic Service-Level Indicator (SLI) on a well-known service type.
               Performance will be computed on the basis of pre-defined metrics.
               SLIs are used to measure and calculate the quality of the Service's
               performance with respect to a single aspect of service quality.
               Exactly one of the following must be set:
               `basic_sli`, `request_based_sli`, `windows_based_sli`  Structure is documented below.
        :param pulumi.Input[str] calendar_period: A calendar period, semantically "since the start of the current
               <calendarPeriod>".
        :param pulumi.Input[str] display_name: Name used for UI elements listing this SLO.
        :param pulumi.Input[float] goal: The fraction of service that must be good in order for this objective
               to be met. 0 < goal <= 0.999
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] request_based_sli: A request-based SLI defines a SLI for which atomic units of
               service are counted directly.
               A SLI describes a good service.
               It is used to measure and calculate the quality of the Service's
               performance with respect to a single aspect of service quality.
               Exactly one of the following must be set:
               `basic_sli`, `request_based_sli`, `windows_based_sli`  Structure is documented below.
        :param pulumi.Input[float] rolling_period_days: A rolling time period, semantically "in the past X days".
               Must be between 1 to 30 days, inclusive.
        :param pulumi.Input[str] service: ID of the service to which this SLO belongs.
        :param pulumi.Input[str] slo_id: The id to use for this ServiceLevelObjective. If omitted, an id will be generated instead.
        :param pulumi.Input[dict] windows_based_sli: A windows-based SLI defines the criteria for time windows.
               good_service is defined based off the count of these time windows
               for which the provided service was of good quality.
               A SLI describes a good service. It is used to measure and calculate
               the quality of the Service's performance with respect to a single
               aspect of service quality.
               Exactly one of the following must be set:
               `basic_sli`, `request_based_sli`, `windows_based_sli`  Structure is documented below.

        The **basic_sli** object supports the following:

          * `latency` (`pulumi.Input[dict]`) - Parameters for a latency threshold SLI.  Structure is documented below.
            * `threshold` (`pulumi.Input[str]`) - A duration string, e.g. 10s.
              Good service is defined to be the count of requests made to
              this service that return in no more than threshold.

          * `locations` (`pulumi.Input[list]`) - An optional set of locations to which this SLI is relevant.
            Telemetry from other locations will not be used to calculate
            performance for this SLI. If omitted, this SLI applies to all
            locations in which the Service has activity. For service types
            that don't support breaking down by location, setting this
            field will result in an error.
          * `methods` (`pulumi.Input[list]`) - An optional set of RPCs to which this SLI is relevant.
            Telemetry from other methods will not be used to calculate
            performance for this SLI. If omitted, this SLI applies to all
            the Service's methods. For service types that don't support
            breaking down by method, setting this field will result in an
            error.
          * `versions` (`pulumi.Input[list]`) - The set of API versions to which this SLI is relevant.
            Telemetry from other API versions will not be used to
            calculate performance for this SLI. If omitted,
            this SLI applies to all API versions. For service types
            that don't support breaking down by version, setting this
            field will result in an error.

        The **request_based_sli** object supports the following:

          * `distributionCut` (`pulumi.Input[dict]`) - Used when good_service is defined by a count of values aggregated in a
            Distribution that fall into a good range. The total_service is the
            total count of all values aggregated in the Distribution.
            Defines a distribution TimeSeries filter and thresholds used for
            measuring good service and total service.  Structure is documented below.
            * `distributionFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              aggregating values to quantify the good service provided.
              Must have ValueType = DISTRIBUTION and
              MetricKind = DELTA or MetricKind = CUMULATIVE.
            * `range` (`pulumi.Input[dict]`) - Range of numerical values. The computed good_service
              will be the count of values x in the Distribution such
              that range.min <= x < range.max. inclusive of min and
              exclusive of max. Open ranges can be defined by setting
              just one of min or max. Summed value `X` should satisfy
              `range.min <= X < range.max` for a good window.  Structure is documented below.
              * `max` (`pulumi.Input[float]`) - max value for the range (inclusive). If not given,
                will be set to "infinity", defining an open range
                ">= range.min"
              * `min` (`pulumi.Input[float]`) - Min value for the range (inclusive). If not given,
                will be set to "-infinity", defining an open range
                "< range.max"

          * `goodTotalRatio` (`pulumi.Input[dict]`) - A means to compute a ratio of `good_service` to `total_service`.
            Defines computing this ratio with two TimeSeries [monitoring filters](https://cloud.google.com/monitoring/api/v3/filters)
            Must specify exactly two of good, bad, and total service filters.
            The relationship good_service + bad_service = total_service
            will be assumed.  Structure is documented below.
            * `badServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              quantifying bad service provided, either demanded service that
              was not provided or demanded service that was of inadequate
              quality. Exactly two of
              good, bad, or total service filter must be defined (where
              good + bad = total is assumed)
              Must have ValueType = DOUBLE or ValueType = INT64 and
              must have MetricKind = DELTA or MetricKind = CUMULATIVE.
            * `goodServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              quantifying good service provided. Exactly two of
              good, bad, or total service filter must be defined (where
              good + bad = total is assumed)
              Must have ValueType = DOUBLE or ValueType = INT64 and
              must have MetricKind = DELTA or MetricKind = CUMULATIVE.
            * `totalServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              quantifying total demanded service. Exactly two of
              good, bad, or total service filter must be defined (where
              good + bad = total is assumed)
              Must have ValueType = DOUBLE or ValueType = INT64 and
              must have MetricKind = DELTA or MetricKind = CUMULATIVE.

        The **windows_based_sli** object supports the following:

          * `goodBadMetricFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
            with ValueType = BOOL. The window is good if any true values
            appear in the window. One of `good_bad_metric_filter`,
            `good_total_ratio_threshold`, `metric_mean_in_range`,
            `metric_sum_in_range` must be set for `windows_based_sli`.
          * `goodTotalRatioThreshold` (`pulumi.Input[dict]`) - Criterion that describes a window as good if its performance is
            high enough. One of `good_bad_metric_filter`,
            `good_total_ratio_threshold`, `metric_mean_in_range`,
            `metric_sum_in_range` must be set for `windows_based_sli`.  Structure is documented below.
            * `basicSliPerformance` (`pulumi.Input[dict]`) - Basic SLI to evaluate to judge window quality.  Structure is documented below.
              * `latency` (`pulumi.Input[dict]`) - Parameters for a latency threshold SLI.  Structure is documented below.
                * `threshold` (`pulumi.Input[str]`) - A duration string, e.g. 10s.
                  Good service is defined to be the count of requests made to
                  this service that return in no more than threshold.

              * `locations` (`pulumi.Input[list]`) - An optional set of locations to which this SLI is relevant.
                Telemetry from other locations will not be used to calculate
                performance for this SLI. If omitted, this SLI applies to all
                locations in which the Service has activity. For service types
                that don't support breaking down by location, setting this
                field will result in an error.
              * `methods` (`pulumi.Input[list]`) - An optional set of RPCs to which this SLI is relevant.
                Telemetry from other methods will not be used to calculate
                performance for this SLI. If omitted, this SLI applies to all
                the Service's methods. For service types that don't support
                breaking down by method, setting this field will result in an
                error.
              * `versions` (`pulumi.Input[list]`) - The set of API versions to which this SLI is relevant.
                Telemetry from other API versions will not be used to
                calculate performance for this SLI. If omitted,
                this SLI applies to all API versions. For service types
                that don't support breaking down by version, setting this
                field will result in an error.

            * `performance` (`pulumi.Input[dict]`) - Request-based SLI to evaluate to judge window quality.  Structure is documented below.
              * `distributionCut` (`pulumi.Input[dict]`) - Used when good_service is defined by a count of values aggregated in a
                Distribution that fall into a good range. The total_service is the
                total count of all values aggregated in the Distribution.
                Defines a distribution TimeSeries filter and thresholds used for
                measuring good service and total service.  Structure is documented below.
                * `distributionFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
                  aggregating values to quantify the good service provided.
                  Must have ValueType = DISTRIBUTION and
                  MetricKind = DELTA or MetricKind = CUMULATIVE.
                * `range` (`pulumi.Input[dict]`) - Range of numerical values. The computed good_service
                  will be the count of values x in the Distribution such
                  that range.min <= x < range.max. inclusive of min and
                  exclusive of max. Open ranges can be defined by setting
                  just one of min or max. Summed value `X` should satisfy
                  `range.min <= X < range.max` for a good window.  Structure is documented below.
                  * `max` (`pulumi.Input[float]`) - max value for the range (inclusive). If not given,
                    will be set to "infinity", defining an open range
                    ">= range.min"
                  * `min` (`pulumi.Input[float]`) - Min value for the range (inclusive). If not given,
                    will be set to "-infinity", defining an open range
                    "< range.max"

              * `goodTotalRatio` (`pulumi.Input[dict]`) - A means to compute a ratio of `good_service` to `total_service`.
                Defines computing this ratio with two TimeSeries [monitoring filters](https://cloud.google.com/monitoring/api/v3/filters)
                Must specify exactly two of good, bad, and total service filters.
                The relationship good_service + bad_service = total_service
                will be assumed.  Structure is documented below.
                * `badServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
                  quantifying bad service provided, either demanded service that
                  was not provided or demanded service that was of inadequate
                  quality. Exactly two of
                  good, bad, or total service filter must be defined (where
                  good + bad = total is assumed)
                  Must have ValueType = DOUBLE or ValueType = INT64 and
                  must have MetricKind = DELTA or MetricKind = CUMULATIVE.
                * `goodServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
                  quantifying good service provided. Exactly two of
                  good, bad, or total service filter must be defined (where
                  good + bad = total is assumed)
                  Must have ValueType = DOUBLE or ValueType = INT64 and
                  must have MetricKind = DELTA or MetricKind = CUMULATIVE.
                * `totalServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
                  quantifying total demanded service. Exactly two of
                  good, bad, or total service filter must be defined (where
                  good + bad = total is assumed)
                  Must have ValueType = DOUBLE or ValueType = INT64 and
                  must have MetricKind = DELTA or MetricKind = CUMULATIVE.

            * `threshold` (`pulumi.Input[float]`) - A duration string, e.g. 10s.
              Good service is defined to be the count of requests made to
              this service that return in no more than threshold.

          * `metricMeanInRange` (`pulumi.Input[dict]`) - Criterion that describes a window as good if the metric's value
            is in a good range, *averaged* across returned streams.
            One of `good_bad_metric_filter`,
            `good_total_ratio_threshold`, `metric_mean_in_range`,
            `metric_sum_in_range` must be set for `windows_based_sli`.
            Average value X of `time_series` should satisfy
            `range.min <= X < range.max` for a good window.  Structure is documented below.
            * `range` (`pulumi.Input[dict]`) - Range of numerical values. The computed good_service
              will be the count of values x in the Distribution such
              that range.min <= x < range.max. inclusive of min and
              exclusive of max. Open ranges can be defined by setting
              just one of min or max. Summed value `X` should satisfy
              `range.min <= X < range.max` for a good window.  Structure is documented below.
              * `max` (`pulumi.Input[float]`) - max value for the range (inclusive). If not given,
                will be set to "infinity", defining an open range
                ">= range.min"
              * `min` (`pulumi.Input[float]`) - Min value for the range (inclusive). If not given,
                will be set to "-infinity", defining an open range
                "< range.max"

            * `timeSeries` (`pulumi.Input[str]`) - A [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              specifying the TimeSeries to use for evaluating window
              quality. The provided TimeSeries must have
              ValueType = INT64 or ValueType = DOUBLE and
              MetricKind = GAUGE.
              Summed value `X` should satisfy
              `range.min <= X < range.max` for a good window.

          * `metricSumInRange` (`pulumi.Input[dict]`) - Criterion that describes a window as good if the metric's value
            is in a good range, *summed* across returned streams.
            Summed value `X` of `time_series` should satisfy
            `range.min <= X < range.max` for a good window.
            One of `good_bad_metric_filter`,
            `good_total_ratio_threshold`, `metric_mean_in_range`,
            `metric_sum_in_range` must be set for `windows_based_sli`.  Structure is documented below.
            * `range` (`pulumi.Input[dict]`) - Range of numerical values. The computed good_service
              will be the count of values x in the Distribution such
              that range.min <= x < range.max. inclusive of min and
              exclusive of max. Open ranges can be defined by setting
              just one of min or max. Summed value `X` should satisfy
              `range.min <= X < range.max` for a good window.  Structure is documented below.
              * `max` (`pulumi.Input[float]`) - max value for the range (inclusive). If not given,
                will be set to "infinity", defining an open range
                ">= range.min"
              * `min` (`pulumi.Input[float]`) - Min value for the range (inclusive). If not given,
                will be set to "-infinity", defining an open range
                "< range.max"

            * `timeSeries` (`pulumi.Input[str]`) - A [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              specifying the TimeSeries to use for evaluating window
              quality. The provided TimeSeries must have
              ValueType = INT64 or ValueType = DOUBLE and
              MetricKind = GAUGE.
              Summed value `X` should satisfy
              `range.min <= X < range.max` for a good window.

          * `windowPeriod` (`pulumi.Input[str]`) - Duration over which window quality is evaluated, given as a
            duration string "{X}s" representing X seconds. Must be an
            integer fraction of a day and at least 60s.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['basic_sli'] = basic_sli
            __props__['calendar_period'] = calendar_period
            __props__['display_name'] = display_name
            if goal is None:
                raise TypeError("Missing required property 'goal'")
            __props__['goal'] = goal
            __props__['project'] = project
            __props__['request_based_sli'] = request_based_sli
            __props__['rolling_period_days'] = rolling_period_days
            if service is None:
                raise TypeError("Missing required property 'service'")
            __props__['service'] = service
            __props__['slo_id'] = slo_id
            __props__['windows_based_sli'] = windows_based_sli
            __props__['name'] = None
        super(Slo, __self__).__init__(
            'gcp:monitoring/slo:Slo',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, basic_sli=None, calendar_period=None, display_name=None, goal=None, name=None, project=None, request_based_sli=None, rolling_period_days=None, service=None, slo_id=None, windows_based_sli=None):
        """
        Get an existing Slo resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] basic_sli: Basic Service-Level Indicator (SLI) on a well-known service type.
               Performance will be computed on the basis of pre-defined metrics.
               SLIs are used to measure and calculate the quality of the Service's
               performance with respect to a single aspect of service quality.
               Exactly one of the following must be set:
               `basic_sli`, `request_based_sli`, `windows_based_sli`  Structure is documented below.
        :param pulumi.Input[str] calendar_period: A calendar period, semantically "since the start of the current
               <calendarPeriod>".
        :param pulumi.Input[str] display_name: Name used for UI elements listing this SLO.
        :param pulumi.Input[float] goal: The fraction of service that must be good in order for this objective
               to be met. 0 < goal <= 0.999
        :param pulumi.Input[str] name: The full resource name for this service. The syntax is:
               projects/[PROJECT_ID_OR_NUMBER]/services/[SERVICE_ID]/serviceLevelObjectives/[SLO_NAME]
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] request_based_sli: A request-based SLI defines a SLI for which atomic units of
               service are counted directly.
               A SLI describes a good service.
               It is used to measure and calculate the quality of the Service's
               performance with respect to a single aspect of service quality.
               Exactly one of the following must be set:
               `basic_sli`, `request_based_sli`, `windows_based_sli`  Structure is documented below.
        :param pulumi.Input[float] rolling_period_days: A rolling time period, semantically "in the past X days".
               Must be between 1 to 30 days, inclusive.
        :param pulumi.Input[str] service: ID of the service to which this SLO belongs.
        :param pulumi.Input[str] slo_id: The id to use for this ServiceLevelObjective. If omitted, an id will be generated instead.
        :param pulumi.Input[dict] windows_based_sli: A windows-based SLI defines the criteria for time windows.
               good_service is defined based off the count of these time windows
               for which the provided service was of good quality.
               A SLI describes a good service. It is used to measure and calculate
               the quality of the Service's performance with respect to a single
               aspect of service quality.
               Exactly one of the following must be set:
               `basic_sli`, `request_based_sli`, `windows_based_sli`  Structure is documented below.

        The **basic_sli** object supports the following:

          * `latency` (`pulumi.Input[dict]`) - Parameters for a latency threshold SLI.  Structure is documented below.
            * `threshold` (`pulumi.Input[str]`) - A duration string, e.g. 10s.
              Good service is defined to be the count of requests made to
              this service that return in no more than threshold.

          * `locations` (`pulumi.Input[list]`) - An optional set of locations to which this SLI is relevant.
            Telemetry from other locations will not be used to calculate
            performance for this SLI. If omitted, this SLI applies to all
            locations in which the Service has activity. For service types
            that don't support breaking down by location, setting this
            field will result in an error.
          * `methods` (`pulumi.Input[list]`) - An optional set of RPCs to which this SLI is relevant.
            Telemetry from other methods will not be used to calculate
            performance for this SLI. If omitted, this SLI applies to all
            the Service's methods. For service types that don't support
            breaking down by method, setting this field will result in an
            error.
          * `versions` (`pulumi.Input[list]`) - The set of API versions to which this SLI is relevant.
            Telemetry from other API versions will not be used to
            calculate performance for this SLI. If omitted,
            this SLI applies to all API versions. For service types
            that don't support breaking down by version, setting this
            field will result in an error.

        The **request_based_sli** object supports the following:

          * `distributionCut` (`pulumi.Input[dict]`) - Used when good_service is defined by a count of values aggregated in a
            Distribution that fall into a good range. The total_service is the
            total count of all values aggregated in the Distribution.
            Defines a distribution TimeSeries filter and thresholds used for
            measuring good service and total service.  Structure is documented below.
            * `distributionFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              aggregating values to quantify the good service provided.
              Must have ValueType = DISTRIBUTION and
              MetricKind = DELTA or MetricKind = CUMULATIVE.
            * `range` (`pulumi.Input[dict]`) - Range of numerical values. The computed good_service
              will be the count of values x in the Distribution such
              that range.min <= x < range.max. inclusive of min and
              exclusive of max. Open ranges can be defined by setting
              just one of min or max. Summed value `X` should satisfy
              `range.min <= X < range.max` for a good window.  Structure is documented below.
              * `max` (`pulumi.Input[float]`) - max value for the range (inclusive). If not given,
                will be set to "infinity", defining an open range
                ">= range.min"
              * `min` (`pulumi.Input[float]`) - Min value for the range (inclusive). If not given,
                will be set to "-infinity", defining an open range
                "< range.max"

          * `goodTotalRatio` (`pulumi.Input[dict]`) - A means to compute a ratio of `good_service` to `total_service`.
            Defines computing this ratio with two TimeSeries [monitoring filters](https://cloud.google.com/monitoring/api/v3/filters)
            Must specify exactly two of good, bad, and total service filters.
            The relationship good_service + bad_service = total_service
            will be assumed.  Structure is documented below.
            * `badServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              quantifying bad service provided, either demanded service that
              was not provided or demanded service that was of inadequate
              quality. Exactly two of
              good, bad, or total service filter must be defined (where
              good + bad = total is assumed)
              Must have ValueType = DOUBLE or ValueType = INT64 and
              must have MetricKind = DELTA or MetricKind = CUMULATIVE.
            * `goodServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              quantifying good service provided. Exactly two of
              good, bad, or total service filter must be defined (where
              good + bad = total is assumed)
              Must have ValueType = DOUBLE or ValueType = INT64 and
              must have MetricKind = DELTA or MetricKind = CUMULATIVE.
            * `totalServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              quantifying total demanded service. Exactly two of
              good, bad, or total service filter must be defined (where
              good + bad = total is assumed)
              Must have ValueType = DOUBLE or ValueType = INT64 and
              must have MetricKind = DELTA or MetricKind = CUMULATIVE.

        The **windows_based_sli** object supports the following:

          * `goodBadMetricFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
            with ValueType = BOOL. The window is good if any true values
            appear in the window. One of `good_bad_metric_filter`,
            `good_total_ratio_threshold`, `metric_mean_in_range`,
            `metric_sum_in_range` must be set for `windows_based_sli`.
          * `goodTotalRatioThreshold` (`pulumi.Input[dict]`) - Criterion that describes a window as good if its performance is
            high enough. One of `good_bad_metric_filter`,
            `good_total_ratio_threshold`, `metric_mean_in_range`,
            `metric_sum_in_range` must be set for `windows_based_sli`.  Structure is documented below.
            * `basicSliPerformance` (`pulumi.Input[dict]`) - Basic SLI to evaluate to judge window quality.  Structure is documented below.
              * `latency` (`pulumi.Input[dict]`) - Parameters for a latency threshold SLI.  Structure is documented below.
                * `threshold` (`pulumi.Input[str]`) - A duration string, e.g. 10s.
                  Good service is defined to be the count of requests made to
                  this service that return in no more than threshold.

              * `locations` (`pulumi.Input[list]`) - An optional set of locations to which this SLI is relevant.
                Telemetry from other locations will not be used to calculate
                performance for this SLI. If omitted, this SLI applies to all
                locations in which the Service has activity. For service types
                that don't support breaking down by location, setting this
                field will result in an error.
              * `methods` (`pulumi.Input[list]`) - An optional set of RPCs to which this SLI is relevant.
                Telemetry from other methods will not be used to calculate
                performance for this SLI. If omitted, this SLI applies to all
                the Service's methods. For service types that don't support
                breaking down by method, setting this field will result in an
                error.
              * `versions` (`pulumi.Input[list]`) - The set of API versions to which this SLI is relevant.
                Telemetry from other API versions will not be used to
                calculate performance for this SLI. If omitted,
                this SLI applies to all API versions. For service types
                that don't support breaking down by version, setting this
                field will result in an error.

            * `performance` (`pulumi.Input[dict]`) - Request-based SLI to evaluate to judge window quality.  Structure is documented below.
              * `distributionCut` (`pulumi.Input[dict]`) - Used when good_service is defined by a count of values aggregated in a
                Distribution that fall into a good range. The total_service is the
                total count of all values aggregated in the Distribution.
                Defines a distribution TimeSeries filter and thresholds used for
                measuring good service and total service.  Structure is documented below.
                * `distributionFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
                  aggregating values to quantify the good service provided.
                  Must have ValueType = DISTRIBUTION and
                  MetricKind = DELTA or MetricKind = CUMULATIVE.
                * `range` (`pulumi.Input[dict]`) - Range of numerical values. The computed good_service
                  will be the count of values x in the Distribution such
                  that range.min <= x < range.max. inclusive of min and
                  exclusive of max. Open ranges can be defined by setting
                  just one of min or max. Summed value `X` should satisfy
                  `range.min <= X < range.max` for a good window.  Structure is documented below.
                  * `max` (`pulumi.Input[float]`) - max value for the range (inclusive). If not given,
                    will be set to "infinity", defining an open range
                    ">= range.min"
                  * `min` (`pulumi.Input[float]`) - Min value for the range (inclusive). If not given,
                    will be set to "-infinity", defining an open range
                    "< range.max"

              * `goodTotalRatio` (`pulumi.Input[dict]`) - A means to compute a ratio of `good_service` to `total_service`.
                Defines computing this ratio with two TimeSeries [monitoring filters](https://cloud.google.com/monitoring/api/v3/filters)
                Must specify exactly two of good, bad, and total service filters.
                The relationship good_service + bad_service = total_service
                will be assumed.  Structure is documented below.
                * `badServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
                  quantifying bad service provided, either demanded service that
                  was not provided or demanded service that was of inadequate
                  quality. Exactly two of
                  good, bad, or total service filter must be defined (where
                  good + bad = total is assumed)
                  Must have ValueType = DOUBLE or ValueType = INT64 and
                  must have MetricKind = DELTA or MetricKind = CUMULATIVE.
                * `goodServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
                  quantifying good service provided. Exactly two of
                  good, bad, or total service filter must be defined (where
                  good + bad = total is assumed)
                  Must have ValueType = DOUBLE or ValueType = INT64 and
                  must have MetricKind = DELTA or MetricKind = CUMULATIVE.
                * `totalServiceFilter` (`pulumi.Input[str]`) - A TimeSeries [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
                  quantifying total demanded service. Exactly two of
                  good, bad, or total service filter must be defined (where
                  good + bad = total is assumed)
                  Must have ValueType = DOUBLE or ValueType = INT64 and
                  must have MetricKind = DELTA or MetricKind = CUMULATIVE.

            * `threshold` (`pulumi.Input[float]`) - A duration string, e.g. 10s.
              Good service is defined to be the count of requests made to
              this service that return in no more than threshold.

          * `metricMeanInRange` (`pulumi.Input[dict]`) - Criterion that describes a window as good if the metric's value
            is in a good range, *averaged* across returned streams.
            One of `good_bad_metric_filter`,
            `good_total_ratio_threshold`, `metric_mean_in_range`,
            `metric_sum_in_range` must be set for `windows_based_sli`.
            Average value X of `time_series` should satisfy
            `range.min <= X < range.max` for a good window.  Structure is documented below.
            * `range` (`pulumi.Input[dict]`) - Range of numerical values. The computed good_service
              will be the count of values x in the Distribution such
              that range.min <= x < range.max. inclusive of min and
              exclusive of max. Open ranges can be defined by setting
              just one of min or max. Summed value `X` should satisfy
              `range.min <= X < range.max` for a good window.  Structure is documented below.
              * `max` (`pulumi.Input[float]`) - max value for the range (inclusive). If not given,
                will be set to "infinity", defining an open range
                ">= range.min"
              * `min` (`pulumi.Input[float]`) - Min value for the range (inclusive). If not given,
                will be set to "-infinity", defining an open range
                "< range.max"

            * `timeSeries` (`pulumi.Input[str]`) - A [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              specifying the TimeSeries to use for evaluating window
              quality. The provided TimeSeries must have
              ValueType = INT64 or ValueType = DOUBLE and
              MetricKind = GAUGE.
              Summed value `X` should satisfy
              `range.min <= X < range.max` for a good window.

          * `metricSumInRange` (`pulumi.Input[dict]`) - Criterion that describes a window as good if the metric's value
            is in a good range, *summed* across returned streams.
            Summed value `X` of `time_series` should satisfy
            `range.min <= X < range.max` for a good window.
            One of `good_bad_metric_filter`,
            `good_total_ratio_threshold`, `metric_mean_in_range`,
            `metric_sum_in_range` must be set for `windows_based_sli`.  Structure is documented below.
            * `range` (`pulumi.Input[dict]`) - Range of numerical values. The computed good_service
              will be the count of values x in the Distribution such
              that range.min <= x < range.max. inclusive of min and
              exclusive of max. Open ranges can be defined by setting
              just one of min or max. Summed value `X` should satisfy
              `range.min <= X < range.max` for a good window.  Structure is documented below.
              * `max` (`pulumi.Input[float]`) - max value for the range (inclusive). If not given,
                will be set to "infinity", defining an open range
                ">= range.min"
              * `min` (`pulumi.Input[float]`) - Min value for the range (inclusive). If not given,
                will be set to "-infinity", defining an open range
                "< range.max"

            * `timeSeries` (`pulumi.Input[str]`) - A [monitoring filter](https://cloud.google.com/monitoring/api/v3/filters)
              specifying the TimeSeries to use for evaluating window
              quality. The provided TimeSeries must have
              ValueType = INT64 or ValueType = DOUBLE and
              MetricKind = GAUGE.
              Summed value `X` should satisfy
              `range.min <= X < range.max` for a good window.

          * `windowPeriod` (`pulumi.Input[str]`) - Duration over which window quality is evaluated, given as a
            duration string "{X}s" representing X seconds. Must be an
            integer fraction of a day and at least 60s.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["basic_sli"] = basic_sli
        __props__["calendar_period"] = calendar_period
        __props__["display_name"] = display_name
        __props__["goal"] = goal
        __props__["name"] = name
        __props__["project"] = project
        __props__["request_based_sli"] = request_based_sli
        __props__["rolling_period_days"] = rolling_period_days
        __props__["service"] = service
        __props__["slo_id"] = slo_id
        __props__["windows_based_sli"] = windows_based_sli
        return Slo(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class GetCaCertsResult:
    """
    A collection of values returned by getCaCerts.
    """
    def __init__(__self__, active_version=None, certs=None, id=None, instance=None, project=None):
        if active_version and not isinstance(active_version, str):
            raise TypeError("Expected argument 'active_version' to be a str")
        __self__.active_version = active_version
        """
        SHA1 fingerprint of the currently active CA certificate.
        """
        if certs and not isinstance(certs, list):
            raise TypeError("Expected argument 'certs' to be a list")
        __self__.certs = certs
        """
        A list of server CA certificates for the instance. Each contains:
        """
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        The provider-assigned unique ID for this managed resource.
        """
        if instance and not isinstance(instance, str):
            raise TypeError("Expected argument 'instance' to be a str")
        __self__.instance = instance
        if project and not isinstance(project, str):
            raise TypeError("Expected argument 'project' to be a str")
        __self__.project = project
class AwaitableGetCaCertsResult(GetCaCertsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetCaCertsResult(
            active_version=self.active_version,
            certs=self.certs,
            id=self.id,
            instance=self.instance,
            project=self.project)

def get_ca_certs(instance=None,project=None,opts=None):
    """
    Get all of the trusted Certificate Authorities (CAs) for the specified SQL database instance. For more information see the
    [official documentation](https://cloud.google.com/sql/)
    and
    [API](https://cloud.google.com/sql/docs/mysql/admin-api/rest/v1beta4/instances/listServerCas).



    :param str instance: The name or self link of the instance.
    :param str project: The ID of the project in which the resource belongs. If `project` is not provided, the provider project is used.
    """
    __args__ = dict()


    __args__['instance'] = instance
    __args__['project'] = project
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('gcp:sql/getCaCerts:getCaCerts', __args__, opts=opts).value

    return AwaitableGetCaCertsResult(
        active_version=__ret__.get('activeVersion'),
        certs=__ret__.get('certs'),
        id=__ret__.get('id'),
        instance=__ret__.get('instance'),
        project=__ret__.get('project'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Connection(pulumi.CustomResource):
    cloud_sql: pulumi.Output[dict]
    """
    Cloud SQL properties.  Structure is documented below.

      * `credential` (`dict`) - Cloud SQL properties.  Structure is documented below.
        * `password` (`str`) - Password for database.  **Note**: This property is sensitive and will not be displayed in the plan.
        * `username` (`str`) - Username for database.

      * `database` (`str`) - Database name.
      * `instance_id` (`str`) - Cloud SQL instance ID in the form project:location:instance.
      * `type` (`str`) - Type of the Cloud SQL database.
    """
    connection_id: pulumi.Output[str]
    """
    Optional connection id that should be assigned to the created connection.
    """
    description: pulumi.Output[str]
    """
    A descriptive description for the connection
    """
    friendly_name: pulumi.Output[str]
    """
    A descriptive name for the connection
    """
    has_credential: pulumi.Output[bool]
    """
    True if the connection has credential assigned.
    """
    location: pulumi.Output[str]
    """
    The geographic location where the connection should reside.
    Cloud SQL instance must be in the same location as the connection
    with following exceptions: Cloud SQL us-central1 maps to BigQuery US, Cloud SQL europe-west1 maps to BigQuery EU.
    Examples: US, EU, asia-northeast1, us-central1, europe-west1. The default value is US.
    """
    name: pulumi.Output[str]
    """
    The resource name of the connection in the form of:
    "projects/{project_id}/locations/{location_id}/connections/{connectionId}"
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    def __init__(__self__, resource_name, opts=None, cloud_sql=None, connection_id=None, description=None, friendly_name=None, location=None, project=None, __props__=None, __name__=None, __opts__=None):
        """
        A connection allows BigQuery connections to external data sources..

        To get more information about Connection, see:

        * [API documentation](https://cloud.google.com/bigquery/docs/reference/bigqueryconnection/rest/v1beta1/projects.locations.connections/create)
        * How-to Guides
            * [Cloud SQL federated queries](https://cloud.google.com/bigquery/docs/cloud-sql-federated-queries)

        > **Warning:** All arguments including `cloud_sql.credential.password` will be stored in the raw
        state as plain-text. [Read more about sensitive data in state](https://www.terraform.io/docs/state/sensitive-data.html).

        ## Example Usage

        ### Bigquery Connection Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp
        import pulumi_random as random

        instance = gcp.sql.DatabaseInstance("instance",
            database_version="POSTGRES_11",
            region="us-central1",
            settings={
                "tier": "db-f1-micro",
            })
        db = gcp.sql.Database("db", instance=instance.name)
        pwd = random.RandomPassword("pwd",
            length=16,
            special=False)
        user = gcp.sql.User("user",
            instance=instance.name,
            password=pwd.result)
        connection = gcp.bigquery.Connection("connection",
            friendly_name="👋",
            description="a riveting description",
            cloud_sql={
                "instance_id": instance.connection_name,
                "database": db.name,
                "type": "POSTGRES",
                "credential": {
                    "username": user.name,
                    "password": user.password,
                },
            })
        ```

        ### Bigquery Connection Full

        ```python
        import pulumi
        import pulumi_gcp as gcp
        import pulumi_random as random

        instance = gcp.sql.DatabaseInstance("instance",
            database_version="POSTGRES_11",
            region="us-central1",
            settings={
                "tier": "db-f1-micro",
            })
        db = gcp.sql.Database("db", instance=instance.name)
        pwd = random.RandomPassword("pwd",
            length=16,
            special=False)
        user = gcp.sql.User("user",
            instance=instance.name,
            password=pwd.result)
        connection = gcp.bigquery.Connection("connection",
            connection_id="my-connection",
            location="US",
            friendly_name="👋",
            description="a riveting description",
            cloud_sql={
                "instance_id": instance.connection_name,
                "database": db.name,
                "type": "POSTGRES",
                "credential": {
                    "username": user.name,
                    "password": user.password,
                },
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] cloud_sql: Cloud SQL properties.  Structure is documented below.
        :param pulumi.Input[str] connection_id: Optional connection id that should be assigned to the created connection.
        :param pulumi.Input[str] description: A descriptive description for the connection
        :param pulumi.Input[str] friendly_name: A descriptive name for the connection
        :param pulumi.Input[str] location: The geographic location where the connection should reside.
               Cloud SQL instance must be in the same location as the connection
               with following exceptions: Cloud SQL us-central1 maps to BigQuery US, Cloud SQL europe-west1 maps to BigQuery EU.
               Examples: US, EU, asia-northeast1, us-central1, europe-west1. The default value is US.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **cloud_sql** object supports the following:

          * `credential` (`pulumi.Input[dict]`) - Cloud SQL properties.  Structure is documented below.
            * `password` (`pulumi.Input[str]`) - Password for database.  **Note**: This property is sensitive and will not be displayed in the plan.
            * `username` (`pulumi.Input[str]`) - Username for database.

          * `database` (`pulumi.Input[str]`) - Database name.
          * `instance_id` (`pulumi.Input[str]`) - Cloud SQL instance ID in the form project:location:instance.
          * `type` (`pulumi.Input[str]`) - Type of the Cloud SQL database.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if cloud_sql is None:
                raise TypeError("Missing required property 'cloud_sql'")
            __props__['cloud_sql'] = cloud_sql
            __props__['connection_id'] = connection_id
            __props__['description'] = description
            __props__['friendly_name'] = friendly_name
            __props__['location'] = location
            __props__['project'] = project
            __props__['has_credential'] = None
            __props__['name'] = None
        super(Connection, __self__).__init__(
            'gcp:bigquery/connection:Connection',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, cloud_sql=None, connection_id=None, description=None, friendly_name=None, has_credential=None, location=None, name=None, project=None):
        """
        Get an existing Connection resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] cloud_sql: Cloud SQL properties.  Structure is documented below.
        :param pulumi.Input[str] connection_id: Optional connection id that should be assigned to the created connection.
        :param pulumi.Input[str] description: A descriptive description for the connection
        :param pulumi.Input[str] friendly_name: A descriptive name for the connection
        :param pulumi.Input[bool] has_credential: True if the connection has credential assigned.
        :param pulumi.Input[str] location: The geographic location where the connection should reside.
               Cloud SQL instance must be in the same location as the connection
               with following exceptions: Cloud SQL us-central1 maps to BigQuery US, Cloud SQL europe-west1 maps to BigQuery EU.
               Examples: US, EU, asia-northeast1, us-central1, europe-west1. The default value is US.
        :param pulumi.Input[str] name: The resource name of the connection in the form of:
               "projects/{project_id}/locations/{location_id}/connections/{connectionId}"
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **cloud_sql** object supports the following:

          * `credential` (`pulumi.Input[dict]`) - Cloud SQL properties.  Structure is documented below.
            * `password` (`pulumi.Input[str]`) - Password for database.  **Note**: This property is sensitive and will not be displayed in the plan.
            * `username` (`pulumi.Input[str]`) - Username for database.

          * `database` (`pulumi.Input[str]`) - Database name.
          * `instance_id` (`pulumi.Input[str]`) - Cloud SQL instance ID in the form project:location:instance.
          * `type` (`pulumi.Input[str]`) - Type of the Cloud SQL database.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["cloud_sql"] = cloud_sql
        __props__["connection_id"] = connection_id
        __props__["description"] = description
        __props__["friendly_name"] = friendly_name
        __props__["has_credential"] = has_credential
        __props__["location"] = location
        __props__["name"] = name
        __props__["project"] = project
        return Connection(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop

